UNIT CompNum;
{$F+,O+,S-,X+,D-,L-,R-}

  (*******************************************************************+
  |       Operace s komplexnimi cisly                                 |
  |       TURBO Pascal v 5.0 -                                        |
  |       Jan Cely, katedra teoreticke fyziky, PF MU Brno             |
  |       ver 2.0 / 02-III-90                                         |
  +*******************************************************************)

INTERFACE

uses DefFloat;

TYPE
  Complex  = RECORD
              Re,Im :FLOAT
             END;
  CompMat2 = ARRAY [1..2,1..2] OF Complex;

CONST
 CompNumVer = 2.0;
 _im1 :Complex=(Re:0.0;Im:1.0);  (* =i *)
 _im0 :Complex=(Re:0  ;Im:0  );  {+++}

PROCEDURE Cnum(Re_p,Im_p:FLOAT; VAR r:Complex);
    (* Vytvoreni kompl.cisla *)

PROCEDURE Cadd(a,b:Complex; VAR r:Complex);
    (* r=a+b *)

PROCEDURE Csub(a,b:Complex; VAR r:Complex);
    (* r=a-b *)

PROCEDURE Cmul(a,b:Complex; VAR r:Complex);
    (* r=a*b *)

PROCEDURE Cdiv(a,b:Complex; VAR r:Complex);
    (* r=a/b *)

PROCEDURE Cexp(a:Complex; VAR r:Complex);
    (* r=exp(a) *)

PROCEDURE Cixp(alfa:FLOAT; VAR r:Complex);
    (* r=exp(i*alfa)  *)

FUNCTION Cnorm(a:Complex):FLOAT;
    (* r=Sqr(|a|) *)

FUNCTION Cabs(a:Complex):FLOAT;
    (* r= |a| *)

FUNCTION Carg(a:Complex):FLOAT;
    (* r=arg(a) z <-pi,pi> v rad *)

FUNCTION Earg(a:Complex):FLOAT;
    (* Vraci argument ve stupnich z <0,360) *)

PROCEDURE Csqr(a:Complex; VAR r:Complex);
    (* r=Sqr(a) *)

PROCEDURE Csqrt(a:Complex; VAR r:Complex);
    (* r=Sqrt(a) ; kladna vetev *)

PROCEDURE RCmul(f:FLOAT; c:Complex; VAR r:Complex);
    (* r=f*c ; nasobeni realnou konstantou *)

{.PA}

PROCEDURE Cwrt(a:Complex; Pozic,DesMist:Integer);
    (* Vystup ve tvaru  (<a.Re>,<a.Im>)
       Pro DesMist<0 vystup v semilogaritmickem tvaru *)

PROCEDURE Cwrtln(a:Complex; Pozic,DesMist:Integer);
    (* Cwrt nasledovane CR/LF *)

PROCEDURE CMulMat2(a,b:CompMat2; VAR c:CompMat2);
    (* Nasobeni kompl.matic 2x2 *)

PROCEDURE RsetCmat2(r11,i11,r12,i12,r21,i21,r22,i22 :FLOAT;
                   VAR c :CompMat2);
    (* Vytvoreni kompl. matice 2x2 zadanim FLOAT.slozek po radcich *)

PROCEDURE CsetCmat2(c11,c12,c21,c22 :Complex;
                    VAR c :CompMat2);
    (* Vytvoreni kompl. matice 2x2 zadanim kompl. prvku po radcich *)

PROCEDURE WrtCmat2(c :CompMat2; Pozic,DesMist :Integer);
    (* Vystup komplexni matice 2x2 s pouzitim Cwrt *)

{.PA}

IMPLEMENTATION

CONST
  pi     = 3.141592653589793238;
  RadDeg = 180.0/pi;               (* 180/pi : radiany na stupne *)

PROCEDURE Cnum(Re_p,Im_p:FLOAT; VAR r:Complex);
 BEGIN
  r.Re:=Re_p; r.Im:=Im_p
 END   (* Cnum *);

PROCEDURE Cadd(a,b:Complex; VAR r:Complex);
 BEGIN
  r.Re:=a.Re + b.Re; r.Im:=a.Im + b.Im
 END   (* Cadd *);

PROCEDURE Csub(a,b:Complex; VAR r:Complex);
 BEGIN
  r.Re:=a.Re - b.Re; r.Im:=a.Im - b.Im
 END   (* Csub *);

PROCEDURE Cmul(a,b:Complex; VAR r:Complex);
 BEGIN
  r.Re:=a.Re*b.Re - a.Im*b.Im;
  r.Im:=a.Im*b.Re + a.Re*b.Im
 END   (* Cmul *);

PROCEDURE Cdiv(a,b:Complex; VAR r:Complex);
VAR p:FLOAT;
 BEGIN
  p:= sqr(b.Re)+sqr(b.Im);
  r.Re:=(a.Re*b.Re + a.Im*b.Im)/p;
  r.Im:=(a.Im*b.Re - a.Re*b.Im)/p
 END   (* Cdiv *);

PROCEDURE Cexp(a:Complex; VAR r:Complex);
 VAR p: FLOAT;
 BEGIN
  p:=exp(a.Re);
  r.Re:=p*cos(a.Im); r.Im:=p*sin(a.Im)
 END   (* Cexp *);

PROCEDURE Cixp(alfa:FLOAT;VAR r:Complex);
 BEGIN
  r.Re:=Cos(alfa); r.Im:=Sin(alfa)
 END; (* Cixp *)

FUNCTION Cnorm(a:Complex):FLOAT;
 BEGIN
  Cnorm:=Sqr(a.Re)+Sqr(a.Im)
 END   (* Cnorm *);

FUNCTION Cabs(a:Complex):FLOAT;
 BEGIN
  Cabs:=Sqrt(Cnorm(a))
 END   (* Cabs *);


{.PA}
FUNCTION Carg(a:Complex):FLOAT;
 VAR
   arc :FLOAT;
 BEGIN
 WITH a DO
  BEGIN
  IF (Re>0) AND (Im=0) THEN Carg:=0
  ELSE
    IF Re=0 THEN
      IF Im>0 THEN Carg:= pi/2
      ELSE Carg:= -pi/2
    ELSE
      BEGIN
        arc:=ArcTan(Im/Re);
        IF (Re>0) THEN  Carg:= arc
        ELSE
          IF (Im>0) THEN Carg:= pi+arc
          ELSE           Carg:= arc-pi;
      END;
  END (*with*)
 END   (* Carg *);

FUNCTION Earg(a:Complex):FLOAT;
 VAR
   arc :FLOAT;
 BEGIN
  WITH a DO
   BEGIN
    IF (re=0) THEN
      IF (im>0) THEN Earg:=90 ELSE Earg:=270
    ELSE
      BEGIN
       arc:=ArcTan(im/re)*RadDeg;
       IF re<0 THEN Earg:=arc+180
       ELSE
         IF im<0 THEN Earg:=arc+360
         ELSE Earg:=arc
      END;
   END (*with*)
 END   (* Earg *);

{.PA}
 PROCEDURE Csqr(a:Complex; VAR r:Complex);
  BEGIN
   r.Re:= Sqr(a.Re) - Sqr(a.Im);
   r.Im:= 2*a.Re*a.Im
  END   (* Csqr *);

 PROCEDURE Csqrt(a:Complex; VAR r:Complex);
   VAR arg,modul :FLOAT;
  BEGIN
    modul:=Sqrt(Cabs(a));
    arg:=Carg(a)/2;
    r.Re:=modul*cos(arg); r.Im:=modul*sin(arg)
  END  (* Csqrt *);

PROCEDURE RCmul(f:FLOAT; c:Complex; VAR r:Complex);
 BEGIN
  r.Re:=f*c.Re; r.Im:=f*c.Im
 END   (* RCmul *);

PROCEDURE Cwrt(a:Complex; Pozic,DesMist:Integer);
 BEGIN
  WITH a DO
   BEGIN
   IF DesMist<0 THEN Write('(',Re:Pozic)
   ELSE Write('(',Re:Pozic:DesMist);
   IF DesMist<0 THEN Write(',',Im:Pozic,')')
   ELSE Write(',',Im:Pozic:DesMist,')')
   END
 END   (* Cwrt *);

PROCEDURE Cwrtln(a:Complex; Pozic,DesMist:Integer);
 BEGIN
  Cwrt(a,Pozic,DesMist); Writeln
 END   (* Cwrtln *);

{.PA}
PROCEDURE CMulMat2(a,b:CompMat2; VAR c:CompMat2);
 VAR
  p1,p2 :Complex;
 BEGIN
  cmul(a[1,1],b[1,1],p1); cmul(a[1,2],b[1,2],p2);
  cadd(p1,p2,c[1,1]);
  cmul(a[1,1],b[1,2],p1); cmul(a[1,2],b[2,2],p2);
  cadd(p1,p2,c[1,2]);
  cmul(a[2,1],b[1,1],p1); cmul(a[2,2],b[2,1],p2);
  cadd(p1,p2,c[2,1]);
  cmul(a[2,1],b[1,2],p1); cmul(a[2,2],b[2,2],p2);
  cadd(p1,p2,c[2,2]);
 END   (* CMulMat2 *);

PROCEDURE RsetCmat2(r11,i11,r12,i12,r21,i21,r22,i22 :FLOAT;
                   VAR c :CompMat2);
 BEGIN
  Cnum(r11,i11,c[1,1]); Cnum(r12,i12,c[1,2]);
  Cnum(r21,i21,c[2,1]); Cnum(r22,i22,c[2,2]);
 END   (* SetCmat2 *);

PROCEDURE CsetCmat2(c11,c12,c21,c22 :Complex;
                    VAR c :CompMat2);
 BEGIN
 c[1,1]:=c11; c[1,2]:=c12; c[2,1]:=c21; c[2,2]:=c22;
 END  (* CsetCmat2 *);

PROCEDURE WrtCmat2(c :CompMat2; Pozic,DesMist :Integer);
 BEGIN
  Cwrt(c[1,1],Pozic,DesMist); Write('    ');
  CwrtLn(c[1,2],Pozic,DesMist);
  Cwrt(c[2,1],Pozic,DesMist); Write('    ');
  CwrtLn(c[2,2],Pozic,DesMist);
 END   (* WrtCmat2 *);

END   (* unit CompNum *).
