{*******************************************************}
{                                                       }
{       Turbo Pascal Version 6.0                        }
{       Turbo Vision Unit                               }
{                                                       }
{       Copyright (c) 1990 Borland International        }
{                                                       }
{  Opraveny chyby v ScanKeyMap a TEditor.ConvertEvent   }
{  podle Bajt 2/1992, 113                               }
{*******************************************************}

UNIT Editor;

{$F+,I-,O+,S-,V-,X+,D-}


INTERFACE

uses Drivers, Objects, Views;

const TabSpaces: byte = 0;   { if >0 then Tabulator=SpaceMaker }
const
  cmSave        = 80;
  cmSaveAs      = 81;
  cmFind        = 82;
  cmReplace     = 83;
  cmSearchAgain = 84;
  cmGotoLine    = 85; {+++}

const
  cmCharLeft    = 500;
  cmCharRight   = 501;
  cmWordLeft    = 502;
  cmWordRight   = 503;
  cmLineStart   = 504;
  cmLineEnd     = 505;
  cmLineUp      = 506;
  cmLineDown    = 507;
  cmPageUp      = 508;
  cmPageDown    = 509;
  cmTextStart   = 510;
  cmTextEnd     = 511;
  cmNewLine     = 512;
  cmBackSpace   = 513;
  cmDelChar     = 514;
  cmDelWord     = 515;
  cmDelStart    = 516;
  cmDelEnd      = 517;
  cmDelLine     = 518;
  cmInsMode     = 519;
  cmStartSelect = 520;
  cmHideSelect  = 521;
  cmIndentMode  = 522;
  cmUpdateTitle = 523;
  cmGotoLineIntern = 524; {+++}

const
  edOutOfMemory   = 0;
  edReadError     = 1;
  edWriteError    = 2;
  edCreateError   = 3;
  edSaveModify    = 4;
  edSaveUntitled  = 5;
  edSaveAs        = 6;
  edFind          = 7;
  edSearchFailed  = 8;
  edReplace       = 9;
  edReplacePrompt = 10;

const
  efCaseSensitive   = $0001;
  efWholeWordsOnly  = $0002;
  efPromptOnReplace = $0004;
  efReplaceAll      = $0008;
  efDoReplace       = $0010;
  efBackupFiles     = $0100;

const
  CIndicator = #2#3;
  CEditor    = #6#7;
  CMemo      = #26#27;

const
  MaxLineLength = 256;

type
  TEditorDialog = function(Dialog: integer; Info: Pointer): Word;

type
  PIndicator = ^TIndicator;
  TIndicator = object(TView)
    Location: TPoint;
    Modified: boolean;
    constructor Init(var Bounds: TRect);
    procedure Draw; virtual;
    function GetPalette: PPalette; virtual;
    procedure SetState(AState: Word; Enable: boolean); virtual;
    procedure SetValue(ALocation: TPoint; AModified: boolean);
  end;

type
  PEditBuffer = ^TEditBuffer;
  TEditBuffer = array[0..65519] of char;

type
  PEditor = ^TEditor;
  TEditor = object(TView)
    HScrollBar: PScrollBar;
    VScrollBar: PScrollBar;
    Indicator: PIndicator;
    Buffer: PEditBuffer;
    BufSize: Word;
    BufLen: Word;
    GapLen: Word;
    SelStart: Word;
    SelEnd: Word;
    CurPtr: Word;
    CurPos: TPoint;
    Delta: TPoint;
    Limit: TPoint;
    DrawLine: integer;
    DrawPtr: Word;
    DelCount: Word;
    InsCount: Word;
    IsValid: boolean;
    CanUndo: boolean;
    Modified: boolean;
    Selecting: boolean;
    Overwrite: boolean;
    AutoIndent: boolean;
    constructor Init(var Bounds: TRect;
      AHScrollBar, AVScrollBar: PScrollBar;
      AIndicator: PIndicator; ABufSize: Word);
    constructor Load(var S: TStream);
    destructor Done; virtual;
    function BufChar(P: Word): char;
    function BufPtr(P: Word): Word;
    procedure ChangeBounds(var Bounds: TRect); virtual;
    procedure ConvertEvent(var Event: TEvent); virtual;
    function CursorVisible: boolean;
    procedure DeleteSelect;
    procedure DoneBuffer; virtual;
    procedure Draw; virtual;
    function GetPalette: PPalette; virtual;
    procedure HandleEvent(var Event: TEvent); virtual;
    procedure InitBuffer; virtual;
    function InsertBuffer(var P: PEditBuffer; Offset, length: Word;
      AllowUndo, SelectText: boolean): boolean;
    function InsertFrom(Editor: PEditor): boolean; virtual;
    function InsertText(text: Pointer; length: Word;
      SelectText: boolean): boolean;
    procedure ScrollTo(X, Y: integer);
    function Search(FindStr: string; Opts: Word): boolean;
    function SetBufSize(NewSize: Word): boolean; virtual;
    procedure SetCmdState(Command: Word; Enable: boolean);
    procedure SetSelect(NewStart, NewEnd: Word; CurStart: boolean);
    procedure SetState(AState: Word; Enable: boolean); virtual;
    procedure Store(var S: TStream);
    procedure TrackCursor(Center: boolean);
    procedure Undo;
    procedure UpdateCommands; virtual;
    function Valid(Command: Word): boolean; virtual;
  private
    LockCount: byte;
    UpdateFlags: byte;
    KeyState: integer;
    function CharPos(P, Target: Word): integer;
    function CharPtr(P: Word; Target: integer): Word;
    function ClipCopy: boolean;
    procedure ClipCut;
    procedure ClipPaste;
    procedure DeleteRange(StartPtr, EndPtr: Word; DelSelect: boolean);
    procedure DoUpdate;
    procedure DoSearchReplace;
    procedure DrawLines(Y, Count: integer; LinePtr: Word);
    procedure FormatLine(var DrawBuf; LinePtr: Word;
      Width: integer; Colors: Word);
    procedure Find;
    function GetMousePtr(Mouse: TPoint): Word;
    function HasSelection: boolean;
    procedure HideSelect;
    function IsClipboard: boolean;
    function LineEnd(P: Word): Word;
    function LineMove(P: Word; Count: integer): Word;
    function LineStart(P: Word): Word;
    procedure Lock;
    procedure NewLine;
    function NextChar(P: Word): Word;
    function NextLine(P: Word): Word;
    function NextWord(P: Word): Word;
    function PrevChar(P: Word): Word;
    function PrevLine(P: Word): Word;
    function PrevWord(P: Word): Word;
    procedure Replace;
    procedure SetBufLen(length: Word);
    procedure SetCurPtr(P: Word; SelectMode: byte);
    procedure StartSelect;
    procedure ToggleInsMode;
    procedure Unlock;
    procedure UpDate(AFlags: byte);
  end;

type
  TMemoData = record
    length: Word;
    Buffer: TEditBuffer;
  end;

type
  PMemo = ^TMemo;
  TMemo = object(TEditor)
    constructor Load(var S: TStream);
    function DataSize: Word; virtual;
    procedure GetData(var Rec); virtual;
    function GetPalette: PPalette; virtual;
    procedure HandleEvent(var Event: TEvent); virtual;
    procedure SetData(var Rec); virtual;
    procedure Store(var S: TStream);
  end;

type
  PFileEditor = ^TFileEditor;
  TFileEditor = object(TEditor)
    FileName: FNameStr;
    constructor Init(var Bounds: TRect;
      AHScrollBar, AVScrollBar: PScrollBar;
      AIndicator: PIndicator; AFileName: FNameStr);
    constructor Load(var S: TStream);
    procedure DoneBuffer; virtual;
    procedure HandleEvent(var Event: TEvent); virtual;
    procedure InitBuffer; virtual;
    function LoadFile: boolean;
    function Save: boolean;
    function SaveAs: boolean;
    function SaveFile: boolean;
    function SetBufSize(NewSize: Word): boolean; virtual;
    procedure Store(var S: TStream);
    procedure UpdateCommands; virtual;
    function Valid(Command: Word): boolean; virtual;
    procedure GotoLine ( Line: integer ); { +++ }
  end;

type
  PEditWindow = ^TEditWindow;
  TEditWindow = object(TWindow)
    Editor: PFileEditor;
    constructor Init(var Bounds: TRect;
      FileName: FNameStr; ANumber: integer);
    constructor Load(var S: TStream);
    procedure close; virtual;
    function GetTitle(MaxSize: integer): TTitleStr; virtual;
    procedure HandleEvent(var Event: TEvent); virtual;
    procedure Store(var S: TStream);
  end;

function DefEditorDialog(Dialog: integer; Info: Pointer): Word;

const
  WordChars: set of char = ['0'..'9', 'A'..'Z', '_', 'a'..'z'];
  EditorDialog: TEditorDialog = DefEditorDialog;
  EditorFlags: Word = efBackupFiles + efPromptOnReplace;
  FindStr: string[80] = '';
  ReplaceStr: string[80] = '';
  Clipboard: PEditor = nil;

type
  TFindDialogRec = record
    Find: string[80];
    Options: Word;
  end;

type
  TReplaceDialogRec = record
    Find: string[80];
    Replace: string[80];
    Options: Word;
  end;

const
  REditor: TStreamRec = (
    ObjType: 70;
    VmtLink: Ofs(TypeOf(TEditor)^);
    Load: @TEditor.Load;
    Store: @TEditor.Store
  );
  RMemo: TStreamRec = (
    ObjType: 71;
    VmtLink: Ofs(TypeOf(TMemo)^);
    Load: @TMemo.Load;
    Store: @TMemo.Store
  );
  RFileEditor: TStreamRec = (
    ObjType: 72;
    VmtLink: Ofs(TypeOf(TFileEditor)^);
    Load: @TFileEditor.Load;
    Store: @TFileEditor.Store
  );
  RIndicator: TStreamRec = (
    ObjType: 73;
    VmtLink: Ofs(TypeOf(TIndicator)^);
    Load: @TIndicator.Load;
    Store: @TIndicator.Store
  );
  REditWindow: TStreamRec = (
    ObjType: 74;
    VmtLink: Ofs(TypeOf(TEditWindow)^);
    Load: @TEditWindow.Load;
    Store: @TEditWindow.Store
  );

procedure RegisterEditors;

IMPLEMENTATION

uses Memory, Buffers, dos, App1;
{!!! App1}

const
  ufUpdate = $01;
  ufLine   = $02;
  ufView   = $04;

const
  smExtend = $01;
  smDouble = $02;

const
  sfSearchFailed = $FFFF;

const
  FirstKeys: array[0..37 * 2] of Word = (37,
    ord(^A), cmWordLeft, ord(^C), cmPageDown,
    ord(^D), cmCharRight, ord(^E), cmLineUp,
    ord(^F), cmWordRight, ord(^g), cmDelChar,
    ord(^H), cmBackSpace, ord(^K), $FF02,
    ord(^L), cmSearchAgain, ord(^M), cmNewLine,
    ord(^O), cmIndentMode, ord(^q), $FF01,
    ord(^R), cmPageUp, ord(^S), cmCharLeft,
    ord(^T), cmDelWord, ord(^U), cmUndo,
    ord(^V), cmInsMode, ord(^X), cmLineDown,
    ord(^Y), cmDelLine, kbLeft, cmCharLeft,
    kbRight, cmCharRight, kbCtrlLeft, cmWordLeft,
    kbCtrlRight, cmWordRight, kbHome, cmLineStart,
    kbEnd, cmLineEnd, kbUp, cmLineUp,
    kbDown, cmLineDown, kbPgUp, cmPageUp,
    kbPgDn, cmPageDown, kbCtrlPgUp, cmTextStart,
    kbCtrlPgDn, cmTextEnd, kbIns, cmInsMode,
    kbDel, cmDelChar, kbShiftIns, cmPaste,
    kbShiftDel, cmCut, kbCtrlIns, cmCopy,
    kbCtrlDel, cmClear);
  QuickKeys: array[0..8 * 2] of Word = (8,
    ord('A'), cmReplace, ord('C'), cmTextEnd,
    ord('D'), cmLineEnd, ord('F'), cmFind,
    ord('H'), cmDelStart, ord('R'), cmTextStart,
    ord('S'), cmLineStart, ord('Y'), cmDelEnd);
  BlockKeys: array[0..5 * 2] of Word = (5,
    ord('B'), cmStartSelect, ord('C'), cmPaste,
    ord('H'), cmHideSelect, ord('K'), cmCopy,
    ord('Y'), cmCut);
  KeyMap: array[0..2] of Pointer = (@FirstKeys, @QuickKeys, @BlockKeys);

function DefEditorDialog(Dialog: integer; Info: Pointer): Word;
begin
  DefEditorDialog := cmCancel;
end;

function Min(X, Y: integer): integer; assembler;
asm
	MOV	AX,X
	CMP	AX,Y
	JLE	@@1
	MOV	AX,Y
@@1:
end;

function Max(X, Y: integer): integer; assembler;
asm
	MOV	AX,X
	CMP	AX,Y
	JGE	@@1
	MOV	AX,Y
@@1:
end;

function MinWord(X, Y: Word): Word; assembler;
asm
	MOV	AX,X
	CMP	AX,Y
	JBE	@@1
	MOV	AX,Y
@@1:
end;

function MaxWord(X, Y: Word): Word; assembler;
asm
	MOV	AX,X
	CMP	AX,Y
	JAE	@@1
	MOV	AX,Y
@@1:
end;

function CountLines(var Buf; Count: Word): integer; assembler;
asm
	LES	DI,Buf
	MOV	CX,Count
	XOR	DX,DX
	MOV	Al,0DH
	CLD
@@1:	JCXZ	@@2
	REPNE	SCASB
	JNE	@@2
	inc	DX
	JMP	@@1
@@2:	MOV	AX,DX
end;

function ScanKeyMap(KeyMap: Pointer; KeyCode: Word): Word; assembler;
asm
	PUSH	DS
	LDS	SI,KeyMap
	MOV	DX,KeyCode
	CLD
	LODSW
	MOV	CX,AX
@@1:	LODSW
	MOV	BX,AX
	LODSW
        CMP     BL,DL
        JNE     @@3
        OR      BH,BH
        JE      @@4
        CMP     BH,DH
        JE      @@4
@@3:	LOOP	@@1
	XOR	AX,AX
@@4:	POP	DS
end;

function Scan(var Block; Size: Word; str: string): Word; assembler;
asm
	PUSH	DS
	LES	DI,Block
	LDS	SI,str
	MOV	CX,Size
	JCXZ	@@3
	CLD
	LODSB
	CMP	Al,1
	JB	@@5
	JA	@@1
	LODSB
	REPNE	SCASB
	JNE	@@3
	JMP	@@5
@@1:	XOR	AH,AH
	MOV	BX,AX
        dec     BX
	MOV	DX,CX
	SUB	DX,AX
	JB	@@3
	LODSB
	inc	DX
	inc	DX
@@2:	dec	DX
	MOV	CX,DX
	REPNE	SCASB
	JNE	@@3
	MOV	DX,CX
	MOV	CX,BX
	REP	CMPSB
	JE	@@4
	SUB	CX,BX
	ADD	SI,CX
	ADD	DI,CX
	inc	DI
	or	DX,DX
	JNE	@@2
@@3:	XOR	AX,AX
	JMP	@@6
@@4:	SUB	DI,BX
@@5:	MOV	AX,DI
	SUB	AX,Word Ptr Block
@@6:	dec	AX
	POP	DS
end;

function IScan(var Block; Size: Word; str: string): Word; assembler;
var
  S: string;
asm
	PUSH	DS
	MOV	AX,SS
	MOV	ES,AX
	LEA	DI,S
	LDS	SI,str
	XOR	AH,AH
	LODSB
	STOSB
	MOV	CX,AX
	MOV	BX,AX
	JCXZ	@@9
@@1:	LODSB
	CMP	Al,'a'
	JB	@@2
	CMP	Al,'z'
	JA	@@2
	SUB	Al,20H
@@2:	STOSB
	LOOP	@@1
	SUB	DI,BX
	LDS	SI,Block
	MOV	CX,Size
	JCXZ	@@8
	CLD
	SUB	CX,BX
	JB	@@8
	inc	CX
@@4:	MOV	AH,ES:[DI]
	and	AH,$DF
@@5:	LODSB
	and	Al,$DF
	CMP	Al,AH
	LOOPNE	@@5
	JNE	@@8
	dec	SI
	MOV	DX,CX
        MOV	CX,BX
@@6:	REPE	CMPSB
	JE	@@10
	MOV	Al,DS:[SI-1]
	CMP	Al,'a'
	JB	@@7
	CMP	Al,'z'
	JA	@@7
	SUB	Al,20H
@@7:	CMP	Al,ES:[DI-1]
	JE	@@6
	SUB	CX,BX
	ADD	SI,CX
	ADD	DI,CX
	inc	SI
	MOV	CX,DX
	JNE	@@4
@@8:	XOR	AX,AX
	JMP	@@11
@@9:    MOV	AX, 1
	JMP	@@11
@@10:	SUB	SI,BX
	MOV	AX,SI
	SUB	AX,Word Ptr Block
	inc	AX
@@11:	dec	AX
	POP	DS
end;

{ TIndicator }

constructor TIndicator.Init(var Bounds: TRect);
var
  R: TRect;
begin
  TView.Init(Bounds);
  GrowMode := gfGrowLoY + gfGrowHiY;
end;

procedure TIndicator.Draw;
var
  Color: byte;
  Frame: char;
  L: array[0..1] of longint;
  S: string[15];
  B: TDrawBuffer;
begin
  if State and sfDragging = 0 then
  begin
    Color := GetColor(1);
    Frame := #205;
  end else
  begin
    Color := GetColor(2);
    Frame := #196;
  end;
  MoveChar(B, Frame, Color, Size.X);
  if Modified then WordRec(B[0]).lo := 15;
  L[0] := Location.Y + 1;
  L[1] := Location.X + 1;
  FormatStr(S, ' %d:%d ', L);
  MoveStr(B[8 - pos(':', S)], S, Color);
  WriteBuf(0, 0, Size.X, 1, B);
end;

function TIndicator.GetPalette: PPalette;
const
  P: string[length(CIndicator)] = CIndicator;
begin
  GetPalette := @P;
end;

procedure TIndicator.SetState(AState: Word; Enable: boolean);
begin
  TView.SetState(AState, Enable);
  if AState = sfDragging then DrawView;
end;

procedure TIndicator.SetValue(ALocation: TPoint; AModified: boolean);
begin
  if (longint(Location) <> longint(ALocation)) or
    (Modified <> AModified) then
  begin
    Location := ALocation;
    Modified := AModified;
    DrawView;
  end;
end;

{ TEditor }

constructor TEditor.Init(var Bounds: TRect;
  AHScrollBar, AVScrollBar: PScrollBar;
  AIndicator: PIndicator; ABufSize: Word);
begin
  TView.Init(Bounds);
  GrowMode := gfGrowHiX + gfGrowHiY;
  Options := Options or ofSelectable;
  EventMask := evMouseDown + evKeyDown + evCommand + evBroadcast;
  ShowCursor;
  HScrollBar := AHScrollBar;
  VScrollBar := AVScrollBar;
  Indicator := AIndicator;
  BufSize := ABufSize;
  CanUndo := true;
  InitBuffer;
  if Buffer <> nil then IsValid := true else
  begin
    EditorDialog(edOutOfMemory, nil);
    BufSize := 0;
  end;
  SetBufLen(0);
end;

constructor TEditor.Load(var S: TStream);
begin
  TView.Load(S);
  GetPeerViewPtr(S, HScrollBar);
  GetPeerViewPtr(S, VScrollBar);
  GetPeerViewPtr(S, Indicator);
  S.read(BufSize, SizeOf(Word));
  S.read(CanUndo, SizeOf(boolean));
  InitBuffer;
  if Buffer <> nil then IsValid := true else
  begin
    EditorDialog(edOutOfMemory, nil);
    BufSize := 0;
  end;
  Lock;
  SetBufLen(0);
end;

destructor TEditor.Done;
begin
  DoneBuffer;
  TView.Done;
end;

function TEditor.BufChar(P: Word): char; assembler;
asm
	LES	DI,Self
	MOV	BX,P
	CMP	BX,ES:[DI].TEditor.CurPtr
	JB	@@1
	ADD	BX,ES:[DI].TEditor.GapLen
@@1:	LES	DI,ES:[DI].TEditor.Buffer
	MOV	Al,ES:[DI+BX]
end;

function TEditor.BufPtr(P: Word): Word; assembler;
asm
	LES	DI,Self
	MOV	AX,P
	CMP	AX,ES:[DI].TEditor.CurPtr
	JB	@@1
	ADD	AX,ES:[DI].TEditor.GapLen
@@1:
end;

procedure TEditor.ChangeBounds(var Bounds: TRect);
begin
  SetBounds(Bounds);
  Delta.X := Max(0, Min(Delta.X, Limit.X - Size.X));
  Delta.Y := Max(0, Min(Delta.Y, Limit.Y - Size.Y));
  UpDate(ufView);
end;

function TEditor.CharPos(P, Target: Word): integer;
var
  pos: integer;
begin
  pos := 0;
  while P < Target do
  begin
    if BufChar(P) = #9 then pos := pos or 7;
    inc(pos);
    inc(P);
  end;
  CharPos := pos;
end;

function TEditor.CharPtr(P: Word; Target: integer): Word;
var
  pos: integer;
begin
  pos := 0;
  while (pos < Target) and (P < BufLen) and (BufChar(P) <> #13) do
  begin
    if BufChar(P) = #9 then pos := pos or 7;
    inc(pos);
    inc(P);
  end;
  if pos > Target then dec(P);
  CharPtr := P;
end;

function TEditor.ClipCopy: boolean;
begin
  ClipCopy := false;
  if (Clipboard <> nil) and (Clipboard <> @Self) then
  begin
    ClipCopy := Clipboard^.InsertFrom(@Self);
    Selecting := false;
    UpDate(ufUpdate);
  end;
end;

procedure TEditor.ClipCut;
begin
  if ClipCopy then DeleteSelect;
end;

procedure TEditor.ClipPaste;
begin
  if (Clipboard <> nil) and (Clipboard <> @Self) then InsertFrom(Clipboard);
end;

procedure TEditor.ConvertEvent(var Event: TEvent);
var
  ShiftState: Byte absolute $40:$17;
  Key: Word;
begin
  if Event.What = evKeyDown then
  begin
    if (ShiftState and $03 <> 0) and
       (Event.ScanCode >= $47) and
       (Event.ScanCode <= $51) then
       Event.CharCode := #0;
    Key := Event.KeyCode;
    if KeyState <> 0 then
    begin
      if (lo(Key) >= $01) and (lo(Key) <= $1A) then inc(Key, $40);
      if (lo(Key) >= $61) and (lo(Key) <= $7A) then dec(Key, $20);
    end;
    Key := ScanKeyMap(KeyMap[KeyState], Key);
    KeyState := 0;
    if Key <> 0 then
      if Hi(Key) = $FF then
      begin
        KeyState := lo(Key);
        ClearEvent(Event);
      end else
      begin
        Event.What := evCommand;
        Event.Command := Key;
      end;
  end;
end;

function TEditor.CursorVisible: boolean;
begin
  CursorVisible := (CurPos.Y >= Delta.Y) and (CurPos.Y < Delta.Y + Size.Y);
end;

procedure TEditor.DeleteRange(StartPtr, EndPtr: Word; DelSelect: boolean);
begin
  if HasSelection and DelSelect then DeleteSelect else
  begin
    SetSelect(CurPtr, EndPtr, true);
    DeleteSelect;
    SetSelect(StartPtr, CurPtr, false);
    DeleteSelect;
  end;
end;

procedure TEditor.DeleteSelect;
begin
  InsertText(nil, 0, false);
end;

procedure TEditor.DoneBuffer;
begin
  if Buffer <> nil then FreeMem(Buffer, BufSize);
end;

procedure TEditor.DoSearchReplace;
var
  I: Word;
  C: TPoint;
begin
  repeat
    I := cmCancel;
    if not Search(FindStr, EditorFlags) then
    begin
      if EditorFlags and (efReplaceAll + efDoReplace) <>
          (efReplaceAll + efDoReplace) then
        EditorDialog(edSearchFailed, nil)
    end
    else if EditorFlags and efDoReplace <> 0 then
    begin
      I := cmYes;
      if EditorFlags and efPromptOnReplace <> 0 then
      begin
        MakeGlobal(Cursor, C);
        I := EditorDialog(edReplacePrompt, Pointer(C));
      end;
      if I = cmYes then
      begin
        Lock;
        InsertText(@ReplaceStr[1], length(ReplaceStr), false);
        TrackCursor(false);
        Unlock;
      end;
    end;
  until (I = cmCancel) or (EditorFlags and efReplaceAll = 0);
end;

procedure TEditor.DoUpdate;
begin
  if UpdateFlags <> 0 then
  begin
    SetCursor(CurPos.X - Delta.X, CurPos.Y - Delta.Y);
    if UpdateFlags and ufView <> 0 then DrawView else
      if UpdateFlags and ufLine <> 0 then
        DrawLines(CurPos.Y - Delta.Y, 1, LineStart(CurPtr));
    if HScrollBar <> nil then
      HScrollBar^.SetParams(Delta.X, 0, Limit.X - Size.X, Size.X div 2, 1);
    if VScrollBar <> nil then
      VScrollBar^.SetParams(Delta.Y, 0, Limit.Y - Size.Y, Size.Y - 1, 1);
    if Indicator <> nil then Indicator^.SetValue(CurPos, Modified);
    if State and sfActive <> 0 then UpdateCommands;
    UpdateFlags := 0;
  end;
end;

procedure TEditor.Draw;
begin
  if DrawLine <> Delta.Y then
  begin
    DrawPtr := LineMove(DrawPtr, Delta.Y - DrawLine);
    DrawLine := Delta.Y;
  end;
  DrawLines(0, Size.Y, DrawPtr);
end;

procedure TEditor.DrawLines(Y, Count: integer; LinePtr: Word);
var
  Color: Word;
  B: array[0..MaxLineLength - 1] of Word;
begin
  Color := GetColor($0201);
  while Count > 0 do
  begin
    FormatLine(B, LinePtr, Delta.X + Size.X, Color);
    WriteBuf(0, Y, Size.X, 1, B[Delta.X]);
    LinePtr := NextLine(LinePtr);
    inc(Y);
    dec(Count);
  end;
end;

procedure TEditor.Find;
var
  FindRec: TFindDialogRec;
begin
  with FindRec do
  begin
    Find := FindStr;
    Options := EditorFlags;
    if EditorDialog(edFind, @FindRec) <> cmCancel then
    begin
      FindStr := Find;
      EditorFlags := Options and not efDoReplace;
      DoSearchReplace;
    end;
  end;
end;

procedure TEditor.FormatLine(var DrawBuf; LinePtr: Word;
  Width: integer; Colors: Word); assembler;
asm
	PUSH	DS
	LDS	BX,Self
	LES	DI,DrawBuf
	MOV	SI,LinePtr
	XOR	DX,DX
	CLD
	MOV	AH,Colors.byte[0]
	MOV	CX,DS:[BX].TEditor.SelStart
	CALL	@@10
	MOV	AH,Colors.byte[1]
	MOV	CX,DS:[BX].TEditor.CurPtr
	CALL	@@10
	ADD	SI,DS:[BX].TEditor.GapLen
	MOV	CX,DS:[BX].TEditor.SelEnd
	ADD	CX,DS:[BX].TEditor.GapLen
	CALL	@@10
	MOV	AH,Colors.byte[0]
	MOV	CX,DS:[BX].TEditor.BufSize
	CALL	@@10
	JMP	@@31
@@10:	SUB	CX,SI
	JA	@@11
	RETN
@@11:	LDS	BX,DS:[BX].TEditor.Buffer
	ADD	SI,BX
	MOV	BX,Width
@@12:	LODSB
	CMP	Al,' '
	JB	@@20
@@13:	STOSW
	inc	DX
@@14:	CMP	DX,BX
	JAE	@@30
	LOOP	@@12
	LDS	BX,Self
	SUB	SI,DS:[BX].TEditor.Buffer.Word[0]
	RETN
@@20:	CMP	Al,0DH
	JE	@@30
	CMP	Al,09H
	JNE	@@13
	MOV	Al,' '
@@21:	STOSW
	inc	DX
	Test	DL,7
	JNE	@@21
	JMP	@@14
@@30:	POP	CX
@@31:	MOV	Al,' '
	MOV	CX,Width
	SUB	CX,DX
	JBE	@@32
	REP	STOSW
@@32:	POP	DS
end;

function TEditor.GetMousePtr(Mouse: TPoint): Word;
begin
  MakeLocal(Mouse, Mouse);
  Mouse.X := Max(0, Min(Mouse.X, Size.X - 1));
  Mouse.Y := Max(0, Min(Mouse.Y, Size.Y - 1));
  GetMousePtr := CharPtr(LineMove(DrawPtr, Mouse.Y + Delta.Y - DrawLine),
    Mouse.X + Delta.X);
end;

function TEditor.GetPalette: PPalette;
const
  P: string[length(CEditor)] = CEditor;
begin
  GetPalette := @P;
end;

procedure TEditor.HandleEvent(var Event: TEvent);
var
  ShiftState: byte absolute $40:$17;
  CenterCursor: boolean;
  SelectMode: byte;
  I: integer;
  NewPtr: Word;
  D, Mouse: TPoint;

procedure CheckScrollBar(P: PScrollBar; var D: integer);
begin
  if (Event.InfoPtr = P) and (P^.Value <> D) then
  begin
    D := P^.Value;
    UpDate(ufView);
  end;
end;

{!!!}
procedure HandleString ( S: string );
var E: TEvent; K: integer;
begin
for K:=1 to length(S) do with E do begin
  What:=evKeyDown; ScanCode:=0; CharCode:=S[K];
  Application^.HandleEvent(E) end
end;

var Pom: integer;
{!!!}

begin
  TView.HandleEvent(Event);
  ConvertEvent(Event);
  CenterCursor := not CursorVisible;
  SelectMode := 0;
  if Selecting or (ShiftState and $03 <> 0) then SelectMode := smExtend;
  case Event.What of
    evMouseDown:
      begin
        if Event.double then SelectMode := SelectMode or smDouble;
        repeat
          Lock;
          if Event.What = evMouseAuto then
          begin
            MakeLocal(Event.Where, Mouse);
            D := Delta;
            if Mouse.X < 0 then dec(D.X);
            if Mouse.X >= Size.X then inc(D.X);
            if Mouse.Y < 0 then dec(D.Y);
            if Mouse.Y >= Size.Y then inc(D.Y);
            ScrollTo(D.X, D.Y);
          end;
          SetCurPtr(GetMousePtr(Event.Where), SelectMode);
          SelectMode := SelectMode or smExtend;
          Unlock;
        until not MouseEvent(Event, evMouseMove + evMouseAuto);
      end;
    evKeyDown:
      case Event.CharCode of
        #9,#32..#255:
          begin
           Lock;
           if (Event.KeyCode=kbTab) and (TabSpaces>0) then {!!!}
               for Pom:=1 to TabSpaces do HandleString(' ')
            else begin
            if Overwrite and not HasSelection then
              if CurPtr <> LineEnd(CurPtr) then SelEnd := NextChar(CurPtr);
            InsertText(@Event.CharCode, 1, false);
            TrackCursor(CenterCursor);
            end;
           Unlock;
          end;
      else
        exit;
      end;
    evCommand:
      case Event.Command of
        cmFind: Find;
        cmReplace: Replace;
        cmSearchAgain: DoSearchReplace;
      else
        begin
          Lock;
          case Event.Command of
            cmCut: ClipCut;
            cmCopy: ClipCopy;
            cmPaste: ClipPaste;
            cmUndo: Undo;
            cmClear: DeleteSelect;
            cmCharLeft: SetCurPtr(PrevChar(CurPtr), SelectMode);
            cmCharRight: SetCurPtr(NextChar(CurPtr), SelectMode);
            cmWordLeft: SetCurPtr(PrevWord(CurPtr), SelectMode);
            cmWordRight: SetCurPtr(NextWord(CurPtr), SelectMode);
            cmLineStart: SetCurPtr(LineStart(CurPtr), SelectMode);
            cmLineEnd: SetCurPtr(LineEnd(CurPtr), SelectMode);
            cmLineUp: SetCurPtr(LineMove(CurPtr, -1), SelectMode);
            cmLineDown: SetCurPtr(LineMove(CurPtr, 1), SelectMode);
 {+++}      cmGotoLineIntern: SetCurPtr(LineMove(CurPtr, Event.InfoInt-(CurPos.Y+1) ), SelectMode);
            cmPageUp: SetCurPtr(LineMove(CurPtr, -(Size.Y - 1)), SelectMode);
            cmPageDown: SetCurPtr(LineMove(CurPtr, Size.Y - 1), SelectMode);
            cmTextStart: SetCurPtr(0, SelectMode);
            cmTextEnd: SetCurPtr(BufLen, SelectMode);
            cmNewLine: NewLine;
            cmBackSpace: DeleteRange(PrevChar(CurPtr), CurPtr, true);
            cmDelChar: DeleteRange(CurPtr, NextChar(CurPtr), true);
            cmDelWord: DeleteRange(CurPtr, NextWord(CurPtr), false);
            cmDelStart: DeleteRange(LineStart(CurPtr), CurPtr, false);
            cmDelEnd: DeleteRange(CurPtr, LineEnd(CurPtr), false);
            cmDelLine: DeleteRange(LineStart(CurPtr), NextLine(CurPtr), false);
            cmInsMode: ToggleInsMode;
            cmStartSelect: StartSelect;
            cmHideSelect: HideSelect;
            cmIndentMode: AutoIndent := not AutoIndent;
          else
            Unlock;
            exit;
          end;
          TrackCursor(CenterCursor);
          Unlock;
        end;
      end;
    evBroadcast:
      case Event.Command of
        cmScrollBarChanged:
          begin
            CheckScrollBar(HScrollBar, Delta.X);
            CheckScrollBar(VScrollBar, Delta.Y);
          end;
      else
        exit;
      end;
  end;
  ClearEvent(Event);
end;

function TEditor.HasSelection: boolean;
begin
  HasSelection := SelStart <> SelEnd;
end;

procedure TEditor.HideSelect;
begin
  Selecting := false;
  SetSelect(CurPtr, CurPtr, false);
end;

procedure TEditor.InitBuffer;
begin
  Buffer := MemAlloc(BufSize);
end;

function TEditor.InsertBuffer(var P: PEditBuffer; Offset, length: Word;
  AllowUndo, SelectText: boolean): boolean;
var
  SelLen, DelLen, SelLines, Lines: Word;
  NewSize: longint;
begin
  InsertBuffer := true;
  Selecting := false;
  SelLen := SelEnd - SelStart;
  if (SelLen = 0) and (length = 0) then exit;
  DelLen := 0;
  if AllowUndo then
    if CurPtr = SelStart then DelLen := SelLen else
      if SelLen > InsCount then DelLen := SelLen - InsCount;
  NewSize := longint(BufLen + DelCount - SelLen + DelLen) + length;
  if NewSize > BufLen + DelCount then
    if (NewSize > $FFF0) or not SetBufSize(NewSize) then
    begin
      EditorDialog(edOutOfMemory, nil);
      InsertBuffer := false;
      exit;
    end;
  SelLines := CountLines(Buffer^[BufPtr(SelStart)], SelLen);
  if CurPtr = SelEnd then
  begin
    if AllowUndo then
    begin
      if DelLen > 0 then Move(Buffer^[SelStart],
        Buffer^[CurPtr + GapLen - DelCount - DelLen], DelLen);
      dec(InsCount, SelLen - DelLen);
    end;
    CurPtr := SelStart;
    dec(CurPos.Y, SelLines);
  end;
  if Delta.Y > CurPos.Y then
  begin
    dec(Delta.Y, SelLines);
    if Delta.Y < CurPos.Y then Delta.Y := CurPos.Y;
  end;
  if length > 0 then Move(P^[Offset], Buffer^[CurPtr], length);
  Lines := CountLines(Buffer^[CurPtr], length);
  inc(CurPtr, length);
  inc(CurPos.Y, Lines);
  DrawLine := CurPos.Y;
  DrawPtr := LineStart(CurPtr);
  CurPos.X := CharPos(DrawPtr, CurPtr);
  if not SelectText then SelStart := CurPtr;
  SelEnd := CurPtr;
  inc(BufLen, length - SelLen);
  dec(GapLen, length - SelLen);
  if AllowUndo then
  begin
    inc(DelCount, DelLen);
    inc(InsCount, length);
  end;
  inc(Limit.Y, Lines - SelLines);
  Delta.Y := Max(0, Min(Delta.Y, Limit.Y - Size.Y));
  if not IsClipboard then Modified := true;
  SetBufSize(BufLen + DelCount);
  if (SelLines = 0) and (Lines = 0) then UpDate(ufLine) else UpDate(ufView);
end;

function TEditor.InsertFrom(Editor: PEditor): boolean;
begin
  InsertFrom := InsertBuffer(Editor^.Buffer,
    Editor^.BufPtr(Editor^.SelStart),
    Editor^.SelEnd - Editor^.SelStart, CanUndo, IsClipboard);
end;

function TEditor.InsertText(text: Pointer; length: Word;
  SelectText: boolean): boolean;
begin
  InsertText := InsertBuffer(PEditBuffer(text),
    0, length, CanUndo, SelectText);
end;

function TEditor.IsClipboard: boolean;
begin
  IsClipboard := Clipboard = @Self;
end;

function TEditor.LineEnd(P: Word): Word; assembler;
asm
	PUSH	DS
	LDS	SI,Self
	LES	BX,DS:[SI].TEditor.Buffer
	MOV	DI,P
	MOV	Al,0DH
	CLD
	MOV	CX,DS:[SI].TEditor.CurPtr
	SUB	CX,DI
	JBE	@@1
	ADD	DI,BX
	REPNE	SCASB
	JE	@@2
	MOV	DI,DS:[SI].TEditor.CurPtr
@@1:	MOV	CX,DS:[SI].TEditor.BufLen
	SUB	CX,DI
	JCXZ	@@4
	ADD	BX,DS:[SI].TEditor.GapLen
	ADD	DI,BX
	REPNE	SCASB
	JNE	@@3
@@2:	dec	DI
@@3:	SUB	DI,BX
@@4:	MOV	AX,DI
	POP	DS
end;

function TEditor.LineMove(P: Word; Count: integer): Word;
var
  pos: integer;
  I: Word;
begin
  I := P;
  P := LineStart(P);
  pos := CharPos(P, I);
  while Count <> 0 do
  begin
    I := P;
    if Count < 0 then
    begin
      P := PrevLine(P);
      inc(Count);
    end else
    begin
      P := NextLine(P);
      dec(Count);
    end;
  end;
  if P <> I then P := CharPtr(P, pos);
  LineMove := P;
end;

function TEditor.LineStart(P: Word): Word; assembler;
asm
	PUSH	DS
	LDS	SI,Self
	LES	BX,DS:[SI].TEditor.Buffer
	MOV	DI,P
	MOV	Al,0DH
	STD
	MOV	CX,DI
	SUB	CX,DS:[SI].TEditor.CurPtr
	JBE	@@1
        ADD	BX,DS:[SI].TEditor.GapLen
	ADD	DI,BX
	dec	DI
	REPNE	SCASB
	JE	@@2
	SUB	BX,DS:[SI].TEditor.GapLen
	MOV	DI,DS:[SI].TEditor.CurPtr
@@1:	MOV	CX,DI
	JCXZ	@@4
	ADD	DI,BX
	dec	DI
	REPNE	SCASB
	JNE	@@3
@@2:	inc	DI
	inc	DI
	SUB	DI,BX
	CMP	DI,DS:[SI].TEditor.CurPtr
	JE	@@4
	CMP	DI,DS:[SI].TEditor.BufLen
	JE	@@4
	CMP	ES:[BX+DI].byte,0AH
	JNE	@@4
	inc	DI
	JMP	@@4
@@3:	XOR	DI,DI
@@4:	MOV	AX,DI
	POP	DS
end;

procedure TEditor.Lock;
begin
  inc(LockCount);
end;

procedure TEditor.NewLine;
const
  CRLF: array[1..2] of char = #13#10;
var
  I, P: Word;
begin
  P := LineStart(CurPtr);
  I := P;
  while (I < CurPtr) and ((Buffer^[I] = ' ') or (Buffer^[I] = #9)) do inc(I);
  InsertText(@CRLF, 2, false);
  if AutoIndent then InsertText(@Buffer^[P], I - P, false);
end;

function TEditor.NextChar(P: Word): Word; assembler;
asm
	PUSH	DS
	LDS	SI,Self
	MOV	DI,P
	CMP	DI,DS:[SI].TEditor.BufLen
	JE	@@2
	inc	DI
	CMP	DI,DS:[SI].TEditor.BufLen
	JE	@@2
	LES	BX,DS:[SI].TEditor.Buffer
	CMP	DI,DS:[SI].TEditor.CurPtr
	JB	@@1
	ADD	BX,DS:[SI].TEditor.GapLen
@@1:	CMP	ES:[BX+DI-1].Word,0A0DH
	JNE	@@2
	inc	DI
@@2:	MOV	AX,DI
	POP	DS
end;

function TEditor.NextLine(P: Word): Word;
begin
  NextLine := NextChar(LineEnd(P));
end;

function TEditor.NextWord(P: Word): Word;
begin
  while (P < BufLen) and (BufChar(P) in WordChars) do
    P := NextChar(P);
  while (P < BufLen) and not (BufChar(P) in WordChars) do
    P := NextChar(P);
  NextWord := P;
end;

function TEditor.PrevChar(P: Word): Word; assembler;
asm
	PUSH	DS
	LDS	SI,Self
	MOV	DI,P
	or	DI,DI
	JE	@@2
	dec	DI
	JE	@@2
	LES	BX,DS:[SI].TEditor.Buffer
	CMP	DI,DS:[SI].TEditor.CurPtr
	JB	@@1
	ADD	BX,DS:[SI].TEditor.GapLen
@@1:	CMP	ES:[BX+DI-1].Word,0A0DH
	JNE	@@2
	dec	DI
@@2:	MOV	AX,DI
	POP	DS
end;

function TEditor.PrevLine(P: Word): Word;
begin
  PrevLine := LineStart(PrevChar(P));
end;

function TEditor.PrevWord(P: Word): Word;
begin
  while (P > 0) and not (BufChar(PrevChar(P)) in WordChars) do
    P := PrevChar(P);
  while (P > 0) and (BufChar(PrevChar(P)) in WordChars) do
    P := PrevChar(P);
  PrevWord := P;
end;

procedure TEditor.Replace;
var
  ReplaceRec: TReplaceDialogRec;
begin
  with ReplaceRec do
  begin
    Find := FindStr;
    Replace := ReplaceStr;
    Options := EditorFlags;
    if EditorDialog(edReplace, @ReplaceRec) <> cmCancel then
    begin
      FindStr := Find;
      ReplaceStr := Replace;
      EditorFlags := Options or efDoReplace;
      DoSearchReplace;
    end;
  end;
end;

procedure TEditor.ScrollTo(X, Y: integer);
begin
  X := Max(0, Min(X, Limit.X - Size.X));
  Y := Max(0, Min(Y, Limit.Y - Size.Y));
  if (X <> Delta.X) or (Y <> Delta.Y) then
  begin
    Delta.X := X;
    Delta.Y := Y;
    UpDate(ufView);
  end;
end;

function TEditor.Search(FindStr: string; Opts: Word): boolean;
var
  I, pos: Word;
begin
  Search := false;
  pos := CurPtr;
  repeat
    if Opts and efCaseSensitive <> 0 then
      I := Scan(Buffer^[BufPtr(pos)], BufLen - pos, FindStr)
    else I := IScan(Buffer^[BufPtr(pos)], BufLen - pos, FindStr);
    if (I <> sfSearchFailed) then
    begin
      inc(I, pos);
      if (Opts and efWholeWordsOnly = 0) or
         not (((I <> 0) and (BufChar(I - 1) in WordChars)) or
              ((I + length(FindStr) <> BufLen) and
               (BufChar(I + length(FindStr)) in WordChars))) then
      begin
        Lock;
        SetSelect(I, I + length(FindStr), false);
        TrackCursor(not CursorVisible);
        Unlock;
        Search := true;
        exit;
      end else pos := I + 1;
    end;
  until I = sfSearchFailed;
end;

procedure TEditor.SetBufLen(length: Word);
begin
  BufLen := length;
  GapLen := BufSize - length;
  SelStart := 0;
  SelEnd := 0;
  CurPtr := 0;
  longint(CurPos) := 0;
  longint(Delta) := 0;
  Limit.X := MaxLineLength;
  Limit.Y := CountLines(Buffer^[GapLen], BufLen) + 1;
  DrawLine := 0;
  DrawPtr := 0;
  DelCount := 0;
  InsCount := 0;
  Modified := false;
  UpDate(ufView);
end;

function TEditor.SetBufSize(NewSize: Word): boolean;
begin
  SetBufSize := NewSize <= BufSize;
end;

procedure TEditor.SetCmdState(Command: Word; Enable: boolean);
var
  S: TCommandSet;
begin
  S := [Command];
  if Enable and (State and sfActive <> 0) then
    EnableCommands(S) else DisableCommands(S);
end;

procedure TEditor.SetCurPtr(P: Word; SelectMode: byte);
var
  Anchor: Word;
begin
  if SelectMode and smExtend = 0 then Anchor := P else
    if CurPtr = SelStart then Anchor := SelEnd else Anchor := SelStart;
  if P < Anchor then
  begin
    if SelectMode and smDouble <> 0 then
    begin
      P := PrevLine(NextLine(P));
      Anchor := NextLine(PrevLine(Anchor));
    end;
    SetSelect(P, Anchor, true);
  end else
  begin
    if SelectMode and smDouble <> 0 then
    begin
      P := NextLine(P);
      Anchor := PrevLine(NextLine(Anchor));
    end;
    SetSelect(Anchor, P, false);
  end;
end;

procedure TEditor.SetSelect(NewStart, NewEnd: Word; CurStart: boolean);
var
  Flags: byte;
  P, L: Word;
begin
  if CurStart then P := NewStart else P := NewEnd;
  Flags := ufUpdate;
  if (NewStart <> SelStart) or (NewEnd <> SelEnd) then
    if (NewStart <> NewEnd) or (SelStart <> SelEnd) then
      Flags := ufView;
  if P <> CurPtr then
  begin
    if P > CurPtr then
    begin
      L := P - CurPtr;
      Move(Buffer^[CurPtr + GapLen], Buffer^[CurPtr], L);
      inc(CurPos.Y, CountLines(Buffer^[CurPtr], L));
      CurPtr := P;
    end else
    begin
      L := CurPtr - P;
      CurPtr := P;
      dec(CurPos.Y, CountLines(Buffer^[CurPtr], L));
      Move(Buffer^[CurPtr], Buffer^[CurPtr + GapLen], L);
    end;
    DrawLine := CurPos.Y;
    DrawPtr := LineStart(P);
    CurPos.X := CharPos(DrawPtr, P);
    DelCount := 0;
    InsCount := 0;
    SetBufSize(BufLen);
  end;
  SelStart := NewStart;
  SelEnd := NewEnd;
  UpDate(Flags);
end;

procedure TEditor.SetState(AState: Word; Enable: boolean);
begin
  TView.SetState(AState, Enable);
  case AState of
    sfActive:
      begin
        if HScrollBar <> nil then HScrollBar^.SetState(sfVisible, Enable);
        if VScrollBar <> nil then VScrollBar^.SetState(sfVisible, Enable);
        if Indicator <> nil then Indicator^.SetState(sfVisible, Enable);
        UpdateCommands;
      end;
    sfExposed:
      if Enable then Unlock;
  end;
end;

procedure TEditor.StartSelect;
begin
  HideSelect;
  Selecting := true;
end;

procedure TEditor.Store(var S: TStream);
begin
  TView.Store(S);
  PutPeerViewPtr(S, HScrollBar);
  PutPeerViewPtr(S, VScrollBar);
  PutPeerViewPtr(S, Indicator);
  S.write(BufSize, SizeOf(Word));
  S.write(CanUndo, SizeOf(boolean));
end;

procedure TEditor.ToggleInsMode;
begin
  Overwrite := not Overwrite;
  SetState(sfCursorIns, not GetState(sfCursorIns));
end;

procedure TEditor.TrackCursor(Center: boolean);
begin
  if Center then
    ScrollTo(CurPos.X - Size.X + 1, CurPos.Y - Size.Y div 2) else
    ScrollTo(Max(CurPos.X - Size.X + 1, Min(Delta.X, CurPos.X)),
      Max(CurPos.Y - Size.Y + 1, Min(Delta.Y, CurPos.Y)));
end;

procedure TEditor.Undo;
var
  length: Word;
begin
  if (DelCount <> 0) or (InsCount <> 0) then
  begin
    SelStart := CurPtr - InsCount;
    SelEnd := CurPtr;
    length := DelCount;
    DelCount := 0;
    InsCount := 0;
    InsertBuffer(Buffer, CurPtr + GapLen - length, length, false, true);
  end;
end;

procedure TEditor.Unlock;
begin
  if LockCount > 0 then
  begin
    dec(LockCount);
    if LockCount = 0 then DoUpdate;
  end;
end;

procedure TEditor.UpDate(AFlags: byte);
begin
  UpdateFlags := UpdateFlags or AFlags;
  if LockCount = 0 then DoUpdate;
end;

procedure TEditor.UpdateCommands;
begin
  SetCmdState(cmUndo, (DelCount <> 0) or (InsCount <> 0));
  if not IsClipboard then
  begin
    SetCmdState(cmCut, HasSelection);
    SetCmdState(cmCopy, HasSelection);
    SetCmdState(cmPaste, (Clipboard <> nil) and (Clipboard^.HasSelection));
  end;
  SetCmdState(cmClear, HasSelection);
  SetCmdState(cmFind, true);
  SetCmdState(cmReplace, true);
  SetCmdState(cmSearchAgain, true);
  SetCmdState(cmGotoLine, true);
end;

function TEditor.Valid(Command: Word): boolean;
begin
  Valid := IsValid;
end;

{ TMemo }

constructor TMemo.Load(var S: TStream);
var
  length: Word;
begin
  TEditor.Load(S);
  S.read(length, SizeOf(Word));
  if IsValid then
  begin
    S.read(Buffer^[BufSize - length], length);
    SetBufLen(length);
  end
  else S.seek(S.GetPos + length);
end;

function TMemo.DataSize: Word;
begin
  DataSize := BufSize + SizeOf(Word);
end;

procedure TMemo.GetData(var Rec);
var
  Data: TMemoData absolute Rec;
begin
  Data.length := BufLen;
  Move(Buffer^, Data.Buffer, CurPtr);
  Move(Buffer^[CurPtr + GapLen], Data.Buffer[CurPtr], BufLen - CurPtr);
  FillChar(Data.Buffer[BufLen], BufSize - BufLen, 0);
end;

function TMemo.GetPalette: PPalette;
const
  P: string[length(CMemo)] = CMemo;
begin
  GetPalette := @P;
end;

procedure TMemo.HandleEvent(var Event: TEvent);
begin
  if (Event.What <> evKeyDown) or (Event.KeyCode <> kbTab) then
    TEditor.HandleEvent(Event);
end;

procedure TMemo.SetData(var Rec);
var
  Data: TMemoData absolute Rec;
begin
  Move(Data.Buffer, Buffer^[BufSize - Data.length], Data.length);
  SetBufLen(Data.length);
end;

procedure TMemo.Store(var S: TStream);
begin
  TEditor.Store(S);
  S.write(BufLen, SizeOf(Word));
  S.write(Buffer^, CurPtr);
  S.write(Buffer^[CurPtr + GapLen], BufLen - CurPtr);
end;

{ TFileEditor }

constructor TFileEditor.Init(var Bounds: TRect;
  AHScrollBar, AVScrollBar: PScrollBar;
  AIndicator: PIndicator; AFileName: FNameStr);
begin
  TEditor.Init(Bounds, AHScrollBar, AVScrollBar, AIndicator, 0);
  if AFileName <> '' then
  begin
    FileName := FExpand(AFileName);
    if IsValid then IsValid := LoadFile;
  end;
end;

constructor TFileEditor.Load(var S: TStream);
var
  SStart, SEnd, Curs: Word;
begin
  TEditor.Load(S);
  S.read(FileName[0], SizeOf(char));
  S.read(FileName[1], length(FileName));
  if IsValid then IsValid := LoadFile;
  S.read(SStart, SizeOf(Word));
  S.read(SEnd, SizeOf(Word));
  S.read(Curs, SizeOf(Word));
  if IsValid and (SEnd <= BufLen) then
  begin
    SetSelect(SStart, SEnd, Curs = SStart);
    TrackCursor(true);
  end;
end;

procedure TFileEditor.DoneBuffer;
begin
  if Buffer <> nil then DisposeBuffer(Buffer);
end;

procedure TFileEditor.HandleEvent(var Event: TEvent);
begin
  TEditor.HandleEvent(Event);
  case Event.What of
    evCommand:
      case Event.Command of
        cmSave: Save;
        cmSaveAs: SaveAs;
      else
        exit;
      end;
  else
    exit;
  end;
  ClearEvent(Event);
end;

procedure TFileEditor.InitBuffer;
begin
  NewBuffer(Pointer(Buffer));
end;

function TFileEditor.LoadFile: boolean;
var
  length: Word;
  FSize: longint;
  F: file;
begin
  LoadFile := false;
  length := 0;
  assign(F, FileName);
  reset(F, 1);
  if IOresult <> 0 then LoadFile := true else
  begin
    FSize := filesize(F);
    if (FSize > $FFF0) or not SetBufSize(Word(FSize)) then
      EditorDialog(edOutOfMemory, nil) else
    begin
      BlockRead(F, Buffer^[BufSize - Word(FSize)], FSize);
      if IOresult <> 0 then EditorDialog(edReadError, @FileName) else
      begin
        LoadFile := true;
        length := FSize;
      end;
    end;
    close(F);
  end;
  SetBufLen(length);
end;

function TFileEditor.Save: boolean;
begin
  if FileName = '' then Save := SaveAs else Save := SaveFile;
end;

function TFileEditor.SaveAs: boolean;
begin
  SaveAs := false;
  if EditorDialog(edSaveAs, @FileName) <> cmCancel then
  begin
    FileName := FExpand(FileName);
    Message(Owner, evBroadcast, cmUpdateTitle, nil);
    SaveAs := SaveFile;
    if IsClipboard then FileName := '';
  end;
end;

function TFileEditor.SaveFile: boolean;
var
  F: file;
  BackupName: FNameStr;
  D: DirStr;
  N: NameStr;
  E: ExtStr;
begin
  SaveFile := false;
  if EditorFlags and efBackupFiles <> 0 then
  begin
    FSplit(FileName, D, N, E);
    BackupName := D + N + '.BAK';
    assign(F, BackupName);
    erase(F);
    assign(F, FileName);
    rename(F, BackupName);
    InOutRes := 0;
  end;
  assign(F, FileName);
  rewrite(F, 1);
  if IOresult <> 0 then EditorDialog(edCreateError, @FileName) else
  begin
    BlockWrite(F, Buffer^, CurPtr);
    BlockWrite(F, Buffer^[CurPtr + GapLen], BufLen - CurPtr);
    if IOresult <> 0 then EditorDialog(edWriteError, @FileName) else
    begin
      Modified := false;
      UpDate(ufUpdate);
      SaveFile := true;
    end;
    close(F);
  end;
end;

function TFileEditor.SetBufSize(NewSize: Word): boolean;
var
  N: Word;
begin
  SetBufSize := false;
  if NewSize > $F000 then NewSize := $FFF0 else
    NewSize := (NewSize + $0FFF) and $F000;
  if NewSize <> BufSize then
  begin
    if NewSize > BufSize then
      if not SetBufferSize(Buffer, NewSize) then exit;
    N := BufLen - CurPtr + DelCount;
    Move(Buffer^[BufSize - N], Buffer^[NewSize - N], N);
    if NewSize < BufSize then SetBufferSize(Buffer, NewSize);
    BufSize := NewSize;
    GapLen := BufSize - BufLen;
  end;
  SetBufSize := true;
end;

procedure TFileEditor.Store(var S: TStream);
begin
  TEditor.Store(S);
  S.write(FileName, length(FileName) + 1);
  S.write(SelStart, SizeOf(Word) * 3);
end;

procedure TFileEditor.UpdateCommands;
begin
  TEditor.UpdateCommands;
  SetCmdState(cmSave, true);
  SetCmdState(cmSaveAs, true);
end;

function TFileEditor.Valid(Command: Word): boolean;
var
  D: integer;
begin
  if Command = cmValid then Valid := IsValid else
  begin
    Valid := true;
    if Modified then
    begin
      if FileName = '' then D := edSaveUntitled else D := edSaveModify;
      case EditorDialog(D, @FileName) of
        cmYes: Valid := Save;
        cmNo: Modified := false;
        cmCancel: Valid := false;
      end;
    end;
  end;
end;

{ TEditWindow }

constructor TEditWindow.Init(var Bounds: TRect;
  FileName: FNameStr; ANumber: integer);
var
  HScrollBar, VScrollBar: PScrollBar;
  Indicator: PIndicator;
  R: TRect;
begin
  TWindow.Init(Bounds, '', ANumber);
  Options := Options or ofTileable;
  R.assign(18, Size.Y - 1, Size.X - 2, Size.Y);
  HScrollBar := new(PScrollBar, Init(R));
  HScrollBar^.Hide;
  Insert(HScrollBar);
  R.assign(Size.X - 1, 1, Size.X, Size.Y - 1);
  VScrollBar := new(PScrollBar, Init(R));
  VScrollBar^.Hide;
  Insert(VScrollBar);
  R.assign(2, Size.Y - 1, 16, Size.Y);
  Indicator := new(PIndicator, Init(R));
  Indicator^.Hide;
  Insert(Indicator);
  GetExtent(R);
  R.Grow(-1, -1);
  Editor := new(PFileEditor, Init(
    R, HScrollBar, VScrollBar, Indicator, FileName));
  Insert(Editor);
end;

constructor TEditWindow.Load(var S: TStream);
begin
  TWindow.Load(S);
  GetSubViewPtr(S, Editor);
end;

procedure TEditWindow.close;
begin
  if Editor^.IsClipboard then Hide else TWindow.close;
end;

function TEditWindow.GetTitle(MaxSize: integer): TTitleStr;
begin
  if Editor^.IsClipboard then GetTitle := 'Clipboard' else
    if Editor^.FileName = '' then GetTitle := 'NONAME' else
      GetTitle := Editor^.FileName;
end;

procedure TEditWindow.HandleEvent(var Event: TEvent);
begin
  TWindow.HandleEvent(Event);
  if (Event.What = evBroadcast) and (Event.Command = cmUpdateTitle) then
  begin
    Frame^.DrawView;
    ClearEvent(Event);
  end;
end;

procedure TEditWindow.Store(var S: TStream);
begin
  TWindow.Store(S);
  PutSubViewPtr(S, Editor);
end;

procedure RegisterEditors;
begin
  RegisterType(REditor);
  RegisterType(RMemo);
  RegisterType(RFileEditor);
  RegisterType(RIndicator);
  RegisterType(REditWindow);
end;

procedure TFileEditor.GotoLine ( Line: integer ); {+++}
var Event: TEvent;
begin
with Event do begin What:=evCommand; Command:=cmGotoLineIntern; InfoInt:=Line end;
TEditor.HandleEvent(Event)
end;


end.
