UNIT ElFun;
{$F+,O+,S-,X+,D-,L-,R-}

  (*--------------------------------------------------------------------+
  |  Elementarni a ruzne pomocne funkce                                 |
  |  ver. 1.0 (XI-89)                                                   |
  |  Turbo Pascal ver. 5.0 -                                            |
  |  Jan Cely, katedra teoreticke fyziky, PF UJEP, Brno                 |
  |                                                                     |
  |  Upravena funkce acos, protoze pri puvodnim naprogramovani pretekal |
  |  zasobnik koprocesoru.   Petr Mikulk                               |
  +--------------------------------------------------------------------*)

INTERFACE

uses DefFloat;

CONST
  ElFunVer = 1.0;

FUNCTION RadToDeg (radiany :FLOAT) :FLOAT;
FUNCTION DegToRad (stupne  :FLOAT) :FLOAT;
  (*----------------------------------------------------------------------+
  |  Prevod radianu na stupne a obracene. Vstupni uhly mimo interval      |
  |  <-2*pi,2*pi>, resp. <-360,360> se prevadi do techto intervalu.       |
  +----------------------------------------------------------------------*)

FUNCTION  PackDeg   (st,min,vt :FLOAT) :FLOAT;
PROCEDURE UnpackDeg (stupne :FLOAT; VAR st,min,vt :FLOAT);
  (*----------------------------------------------------------------------+
  |  PackDeg provadi prevod ze stupnu,minut a vterin na stupne            |
  |  UnpackDeg provadi opacny prevod (st a min maji jen celociselnou cast)|
  +----------------------------------------------------------------------*)

FUNCTION Power(n:INTEGER; x:FLOAT) :FLOAT;
  (*----------------------------------------------------------------------+
  |  Vraci n-tou mocninu x; n kladne i zaporne                            |
  +----------------------------------------------------------------------*)

FUNCTION N_Root(n:INTEGER; x,est,eps:FLOAT) :FLOAT;
  (*----------------------------------------------------------------------+
  |  Pocita x^(1/n) Halleyovou iteracni metodou; n kladne i zaporne       |
  |  est je pocatecni odhad (zpravidla vyhovuje est=1)                    |
  |         pro est=0 se dosadi est:=1.0                                  |
  |  eps je relativni chyba vysledku;                                     |
  |         iteracni proces konci pro |xnew-xold|<eps*xnew                |
  |         pro eps=0 se dosadi eps:=1e-12 pri N- a eps:=1e-16 pri N+     |
  +----------------------------------------------------------------------*)



FUNCTION Log10 (x:FLOAT) :FLOAT;
FUNCTION Alog10(x:FLOAT) :FLOAT;
  (*----------------------------------------------------------------------+
  |  Dekadicky logaritmus a antilogaritmus (=10^x) z x                    |
  +----------------------------------------------------------------------*)

FUNCTION asin (x :FLOAT): FLOAT;
FUNCTION acos (x :FLOAT): FLOAT;
  (*----------------------------------------------------------------------+
  |  arcsin(x) a arccos(x)                                                |
  |  Aproximace s chybou |eps| < 2.0e-8                                   |
  +----------------------------------------------------------------------*)

FUNCTION sinh (x :FLOAT): FLOAT;
FUNCTION cosh (x :FLOAT): FLOAT;
FUNCTION tanh (x :FLOAT): FLOAT;
  (*----------------------------------------------------------------------+
  |  Hyperbolicke funkce                                                  |
  +----------------------------------------------------------------------*)

FUNCTION asinh (x :FLOAT): FLOAT;
FUNCTION acosh (x :FLOAT): FLOAT;
FUNCTION atanh (x :FLOAT): FLOAT;
  (*----------------------------------------------------------------------+
  |  Hyperbolometricke (inverzni hyperbolicke) funkce                     |
  |  asinh(x)  pro vsechna x; acosh(x) pro x>=1 ; atanh(x) pro |x|<1      |
  +----------------------------------------------------------------------*)


IMPLEMENTATION

CONST
  pi     = 3.1415926535897932385;
  TwoPi  = 2.0*pi;
  pi2    = pi/2;
  log10e = 0.43429448190325182765;
  Ln10   = 2.3025850929940456840;
  RtD    = 180.0/pi;
  DtR    = pi/180.0;


PROCEDURE Error (erno :BYTE);
  (* Ohlasi chybu na displeji a provede Halt *)
  BEGIN
    Writeln; Write(#7,' * * * ');
    CASE erno OF
      1 : Writeln(' Chybny argument v asin nebo acos');
      2 : Writeln(' V acosh(x) je x<1.0');
      3 : Writeln(' V atanh(x) je |x|>=1.0');
    END;
    Writeln; Halt(1);
  END;

FUNCTION RadToDeg;
  VAR s :FLOAT;
  BEGIN
    s:= radiany * RtD;
    IF Abs(s)>360.0 THEN s:= Frac(s/360.0)*360.0;
    RadToDeg:=s;
  END; (* RadToDeg *)

FUNCTION DegToRad;
  VAR s :FLOAT;
  BEGIN
    s:= stupne * DtR;
    IF Abs(s)>TwoPi THEN s:= Frac(s/TwoPi)*TwoPi;
    DegToRad:= s;
  END; (* DegToRad *)

FUNCTION PackDeg;
  BEGIN
    PackDeg:= st + min/60.0 + vt/3600.0;
  END; (* PackDeg *)

PROCEDURE UnpackDeg;
  VAR p :FLOAT;
  BEGIN
    st := Int(stupne);
    p  := Frac(stupne)*60.0;
    min:= Int(p);
    vt := Frac(p)*60;
  END; (* UnpackDeg *)


FUNCTION Log10;
  BEGIN
    Log10:= Ln(x)*log10e
  END;  (* Log10 *)

FUNCTION Alog10;
  BEGIN
    Alog10:= Exp(x*Ln10)
  END;  (* Alog10 *)


FUNCTION acos;
  CONST
    a0 = 1.5707963050; a1 = -0.2145988016;
    a2 = 0.0889789874; a3 = -0.0501743046;
    a4 = 0.0308918810; a5 = -0.0170881256;
    a6 = 0.0066700901; a7 = -0.0012624911;
  VAR
    s,y :FLOAT;
  BEGIN
   IF X=0 THEN ACOS:=PI2 ELSE BEGIN
    y:=Abs(x);
    IF y>1 THEN Error(1);
{    s:=Sqrt(1-y)*(a0+y*(a1+y*(a2+y*(a3+y*(a4+y*(a5+y*(a6+y*a7))))))); }
{    ^ toto bylo puvodne a pretekal zasobnik koprocesoru }
    s:=a4+y*(a5+y*(a6+y*a7));
    s:=a0+y*(a1+y*(a2+y*(a3+y*s)));
    s:=Sqrt(1-y)*s;
    IF x<0 THEN s:= pi - s;
    acos:= s; END;
  END; (*acos*)

FUNCTION asin;
  BEGIN
   IF X=0 THEN ASIN:=0 ELSE
    asin:= pi2 - acos(x);
  END; (*asin*)

FUNCTION Power(n:Integer; x:FLOAT):FLOAT;
  VAR y:FLOAT;
      i:Integer;
  BEGIN
   i:=Abs(n); y:=1;
   WHILE i>0 DO BEGIN
     IF Odd(i) THEN y:=y*x; i:=i DIV 2; x:= Sqr(x)
   END;
   IF n>=0 THEN Power:=y ELSE Power:=1/y
  END; (* Power *)

FUNCTION N_Root(n:Integer;x,est,eps:FLOAT):FLOAT;
  VAR np1,nm1,xn,xold,xnew,r:FLOAT;
  BEGIN
   IF est=0.0 THEN est:=1;
   IF eps=0.0 THEN {$IFOPT N+}  eps:= 1e-16;
                   {$ELSE}      eps:= 1e-12;
                   {$ENDIF}
   IF n IN [0,1] THEN
    CASE n OF
     0 : r:= 1;
     1 : r:= x;
    END
   ELSE
    IF x=0 THEN r:= 0
    ELSE
     BEGIN
      np1:=Abs(n)+1.0; nm1:=Abs(n)-1.0; xnew:=est;
      REPEAT
       xold:=xnew; xn:=Power(Abs(n),xold);
       xnew:=((nm1*xn+np1*x)/(np1*xn+nm1*x))*xold;
      UNTIL Abs(xnew-xold)<=eps;
      r:=xnew
     END;
   IF (n>0) OR (r=0) THEN N_Root:=r
   ELSE N_Root:=1/r;
  END; (* N_Root *)


FUNCTION sinh (x :FLOAT): FLOAT;
  VAR t :FLOAT;
  BEGIN
   t:=Exp(x);
   sinh:= 0.5*(t-1/t)
  END;  (* sinh *)

FUNCTION cosh (x :FLOAT): FLOAT;
  VAR t :FLOAT;
  BEGIN
   t:=Exp(x);
   cosh:= 0.5*(t+1/t)
  END;  (* cosh *)

FUNCTION tanh (x :FLOAT): FLOAT;
  VAR s,t :FLOAT;
  BEGIN
   s:=Exp(x); t:=1/s;
   tanh:= (s-t)/(s+t);
  END;  (* tanh *)

FUNCTION asinh (x :FLOAT): FLOAT;
  VAR t: FLOAT;
  BEGIN
    t:= x+Sqrt(Sqr(x)+1);
    IF t<0 THEN
      asinh:= 0.0
    ELSE
      asinh:= Ln(t);
  END;  (* asinh *)

FUNCTION acosh (x :FLOAT): FLOAT;
  BEGIN
   IF x<1.0 THEN Error(2);
   acosh:= Ln(x+Sqrt(Sqr(x)-1.0));
  END;  (* acosh *)

FUNCTION atanh (x :FLOAT): FLOAT;
  BEGIN
   IF (Abs(x)>=1.0) OR ((1.0-x)=0.0) THEN Error(3);
   atanh:= 0.5*Ln((1.0+x)/(1.0-x));
  END;  (* atanh *)



END.  (* ElFun *)

