{$A+,B-,D-,E+,F+,G+,I-,L-,N-,O+,R-,S-,V-,X-}
{************************************************}
{                                                }
{   Copyright (c) 1990 by Borland International  }
{             (c) 1991 by Petr Mikulik           }
{                                                }
{************************************************}

UNIT Kalk;

INTERFACE

uses Drivers, Objects, Views, Dialogs, App1;

type

  TCalcState = (csFirst, csValid, csError);

  PCalcDisplay = ^TCalcDisplay;
  TCalcDisplay = object(TView)
    Status: TCalcState;
    Number: string[15];
    Sign: char;
    Operator: char;
    Operand: real;
    constructor Init(var Bounds: TRect);
    constructor Load(var S: TStream);
    procedure CalcKey(Key: char);
    procedure Clear;
    procedure Draw; virtual;
    function GetPalette: PPalette; virtual;
    procedure HandleEvent(var Event: TEvent); virtual;
    procedure Store(var S: TStream);
  end;

  PCalculator = ^TCalculator;
  TCalculator = object(TDialog)
    constructor Init;
  end;

  PCalculatorW = ^TCalculatorW;
  TCalculatorW = object ( TCalculator )
     constructor Init ( X, Y: integer );
  end;

  PCalculatorWA = ^TCalculatorWA;
  TCalculatorWA = object(TCalculatorW)
    procedure HandleEvent ( var E: TEvent ); virtual;
    { movable calculator by arrows }
  end;

  PCalculatorWAE = ^TCalculatorWAE;
  TCalculatorWAE = object ( TCalculatorWA )
    procedure HandleEvent ( var E: TEvent ); virtual;
    { movable + ESC = close }
  end;

  PCalculatorWAQ = ^TCalculatorWAQ;
  TCalculatorWAQ = object ( TCalculatorWA )
    procedure HandleEvent ( var E: TEvent ); virtual;
    { movable + ESC = done application }
  end;

const
  RCalcDisplay: TStreamRec = (
     ObjType: 10040;
     VmtLink: Ofs(TypeOf(TCalcDisplay)^);
     Load:    @TCalcDisplay.Load;
     Store:   @TCalcDisplay.Store
  );
  RCalculator: TStreamRec = (
     ObjType: 10041;
     VmtLink: Ofs(TypeOf(TCalculator)^);
     Load:    @TCalculator.Load;
     Store:   @TCalculator.Store
  );

procedure RegisterCalc;

const OldX: byte = 5; { predchozi pozice }
      OldY: byte = 3;

IMPLEMENTATION

const
  cmCalcButton = 100;

constructor TCalcDisplay.Init(var Bounds: TRect);
begin
  TView.Init(Bounds);
  Options := Options or ofSelectable;
  EventMask := evKeyDown + evBroadcast;
  Clear;
end;

constructor TCalcDisplay.Load(var S: TStream);
begin
  TView.Load(S);
  S.read(Status, SizeOf(Status) + SizeOf(Number) + SizeOf(Sign) +
    SizeOf(Operator) + SizeOf(Operand));
end;

procedure TCalcDisplay.CalcKey(Key: char);
var
  R: real;

procedure Error;
begin
  Status := csError;
  Number := 'Error';
  Sign := ' ';
end;

procedure SetDisplay(R: real);
var
  S: string[63];
begin
  str(R: 0: 10, S);
  if S[1] <> '-' then Sign := ' ' else
  begin
    Delete(S, 1, 1);
    Sign := '-';
  end;
  if length(S) > 15 + 1 + 10 then Error
  else
  begin
    while S[length(S)] = '0' do dec(S[0]);
    if S[length(S)] = '.' then dec(S[0]);
    Number := S;
  end;
end;

procedure GetDisplay(var R: real);
var
  E: integer;
begin
  val(Sign + Number, R, E);
end;

procedure CheckFirst;
begin
  if Status = csFirst then
  begin
    Status := csValid;
    Number := '0';
    Sign := ' ';
  end;
end;

begin
  Key := UpCase(Key);
  if (Status = csError) and (Key <> 'C') then Key := ' ';
  case Key of
    '0'..'9':
      begin
        CheckFirst;
        if Number = '0' then Number := '';
        Number := Number + Key;
      end;
    '.':
      begin
        CheckFirst;
        if pos('.', Number) = 0 then Number := Number + '.';
      end;
    #8, #27:
      begin
        CheckFirst;
        if length(Number) = 1 then Number := '0' else dec(Number[0]);
      end;
    '_', #241:
      if Sign = ' ' then Sign := '-' else Sign := ' ';
    '+', '-', '*', '/', '=', '%', #13:
      begin
        if Status = csValid then
        begin
          Status := csFirst;
          GetDisplay(R);
          if Key = '%' then
            case Operator of
              '+', '-': R := Operand * R / 100;
              '*', '/': R := R / 100;
            end;
          case Operator of
            '+': SetDisplay(Operand + R);
            '-': SetDisplay(Operand - R);
            '*': SetDisplay(Operand * R);
            '/': if R = 0 then Error else SetDisplay(Operand / R);
          end;
        end;
        Operator := Key;
        GetDisplay(Operand);
      end;
    'C':
      Clear;
  end;
  DrawView;
end;

procedure TCalcDisplay.Clear;
begin
  Status := csFirst;
  Number := '0';
  Sign := ' ';
  Operator := '=';
end;

procedure TCalcDisplay.Draw;
var
  Color: byte;
  I: integer;
  B: TDrawBuffer;
begin
  Color := GetColor(1);
  I := Size.X - length(Number) - 2;
  MoveChar(B, ' ', Color, Size.X);
  MoveChar(B[I], Sign, Color, 1);
  MoveStr(B[I + 1], Number, Color);
  WriteBuf(0, 0, Size.X, 1, B);
end;

function TCalcDisplay.GetPalette: PPalette;
const
  P: string[1] = #19;
begin
  GetPalette := @P;
end;

procedure TCalcDisplay.HandleEvent(var Event: TEvent);
begin
  TView.HandleEvent(Event);
  case Event.What of
    evKeyDown:
      begin
        CalcKey(Event.CharCode);
        ClearEvent(Event);
      end;
    evBroadcast:
      if Event.Command = cmCalcButton then
      begin
        CalcKey(PButton(Event.InfoPtr)^.Title^[1]);
        ClearEvent(Event);
      end;
  end;
end;

procedure TCalcDisplay.Store(var S: TStream);
begin
  TView.Store(S);
  S.write(Status, SizeOf(Status) + SizeOf(Number) + SizeOf(Sign) +
    SizeOf(Operator) + SizeOf(Operand));
end;

{ TCalculator }

constructor TCalculator.Init;
const
  KeyChar: array[0..19] of char = 'C'#27'%'#241'789/456*123-0.=+';
var
  I: integer;
  P: PView;
  R: TRect;
begin
  R.assign(5, 3, 29, 18);
  TDialog.Init(R, 'Calculator');
  Options := Options or ofFirstClick;
  for I := 0 to 19 do
  begin
    R.A.X := (I mod 4) * 5 + 2;
    R.A.Y := (I div 4) * 2 + 4;
    R.B.X := R.A.X + 5;
    R.B.Y := R.A.Y + 2;
    P := new(PButton, Init(R, KeyChar[I], cmCalcButton,
      bfNormal + bfBroadcast));
    P^.Options := P^.Options and not ofSelectable;
    Insert(P);
  end;
  R.assign(3, 2, 21, 3);
  Insert(new(PCalcDisplay, Init(R)));
end;


constructor TCalculatorW.Init ( X,Y: integer );
begin
TCalculator.Init;
MoveTo( X,Y )
end;

procedure TCalculatorWA.HandleEvent(var E: TEvent);
begin
if (E.What=evKeyDown) and ((E.KeyCode=kbUp) or (E.KeyCode=kbDown)
   or (E.KeyCode=kbLeft) or (E.KeyCode=kbRight)) then
   begin Application^.Putcm(cmResize); ClearEvent(E) end
   else TCalculatorW.HandleEvent(E);
end;

procedure TCalculatorWAE.HandleEvent(var E: TEvent);
begin
if ((E.What=evKeyDown) and (E.KeyCode=kbEsc))
     then begin E.What:=evCommand; E.Command:=cmClose;
     Application^.HandleEvent(E) end
     else TCalculatorWA.HandleEvent(E)
{
if ((E.What=evKeyDown) and (E.KeyCode=kbEsc))
     then begin Application^.Putcm(cmClose); ClearEvent(E) end
     else TCalculatorWA.HandleEvent(E)
}
end;

procedure TCalculatorWAQ.HandleEvent(var E: TEvent);
begin
if ((E.What=evCommand) and (E.Command=cmClose))
     then begin Application^.Putcm(cmQuit); ClearEvent(E) end;
if ((E.What=evKeyDown) and (E.KeyCode=kbEsc))
     then begin Application^.Putcm(cmQuit); ClearEvent(E) end
     else TCalculatorWA.HandleEvent(E);
end;

procedure RegisterCalc;
begin
  RegisterType(RCalcDisplay);
  RegisterType(RCalculator);
end;

end.
