UNIT pmparser;

{$F+,O+,S-,X+,D-,L-,R-}
{ $F+,O+,S-,X+,D+,L+,R+}

(* PMParser.pas: mathematical formula parser
   c Petr Mikulk - Brno - 1994   (mikulik@sci.muni.cz)

  22. 1.1996: added floor, ceil and round functions
  17.11.1995: corrected $nm problem.
*)

{ This code was rewritten from my previous C++ code. Maybe the code
  is not clear, but it works.
  This program is public domain if my name remains above.
  It should be distributed together with the demo program.
  New versions (if any) could be found on
  ftp.muni.cz:pub/muni.cz/physics (accessible also via gopher.muni.cz)
  }


INTERFACE

{ error_in_formula handling: }
var ParserError: byte;    { global error indicator of parser }
const peOK=0;
      peIdentifier=1;
      peOperator=2;
      peParentheses=3;
      peDollar=4;


type PChar = ^ char;
var ParserIndex: pchar;
  { address of char where an error occured if ParserError<>0 }


procedure FormulaParserPreProcess ( var S: string );
{ preprocessing. It converts string S to lower case and it removes spaces
  It should be called before the following two routines }


function FormulaParser ( var S: string ): real;
{ you give string in, like "7.4*sin(pi/3*5.5)+(1-3)*3",
  and it returns the result }


function FormulaParserDollars
  ( var S: string; var Dollars {: array[0..10921] of real} ): real;
{  like the previous one. Moreover, you can address the elements
   of the Dollars array by  $n  names ($0==Dollars[0], $1==Dollars[1] etc.)
   Moreover, keywords x,y,z are equivalents to $1, $2, $3
   Example: "sin( ($1+$2)*deg2rad )*z + 1"
}

{
   For the list of the known constants and functions, look below.
   This should be known:
   Constants:
     "pi","e","rad2deg","deg2rad","rad2sec","sec2rad"
   Functions of one parameter:
     "sin","cos","tan","log","log10","exp","abs","fabs",
     "sqrt","sqr","asin","acos","atan","sinh","cosh","tanh",
     "sinc","sgn","step","int","floor","ceil","round"
     Note: sinc(x)=sin(x)/x
           step(x)= x>=0 ? 1:0 (Heaviside function)
   Functions of two parameters:
     "pow"  pow(x,y)=x**y
     "eq"   eq(x,y) = x==y
     "mod"  mod(x,y) = x modulo y
   Functions of three parameters:
     "ifno0"(expr,x,y):   expr!=0 ? x : y
     "ifpos"(expr,x,y):   expr>0 ?  x : y
     "ifpos0"(expr,x,y):  expr>=0 ? x : y
     "ifneg"(expr,x,y):   expr<0 ?  x : y
     "ifneg0"(expr,x,y):  expr<=0 ? x : y
   Note: division 0/0 gives 1, division x/0 gives sgn(x)*1e35,
     log,log10 of x<=0 gives -1e35
}


function FormulaParserGetMaxParam ( var S: string ): integer;
{ returns max. nb. of parameter ($n) in the formula S
  E.g. FormulaParserGetMaxParam( 'sin($4)+$1' ) returns 4 }


IMPLEMENTATION

var ParserUseDollars, ParserUsexyz: boolean;
type PReal = ^ TArrayReal;
     TArrayReal = array [0..10921] of real;
var ParserDollars: PReal;

const ParserConstName: string[50] =
       'pi'#0'e'#0'rad2deg'#0'deg2rad'#0'rad2sec'#0'sec2rad'#0#0;
      ParserConst: array [0..5] of real =
       ( 3.14159265358979323846,2.71828182845904523536,
         180./3.14159265358979323846,3.14159265358979323846/180,
         6480000./3.14159265358979323846,3.14159265358979323846/6480000.
       );


function parser_sinc ( x: real ): real;
 begin if (x=0) then parser_sinc:=1 else parser_sinc:=sin(x)/x end;
function parser_sgn ( x: real ): real;
  begin if x=0 then parser_sgn:=0 else if x>0 then parser_sgn:=1
    else parser_sgn:=-1 end;
function parser_log ( x: real ): real;
  begin if x>0 then parser_log:=ln(x) else parser_log:=-1e35 end;
const log10_e = 0.434294481903;
const ln_10   = 2.302585092994;
{ alog10(x:real): real; alog10:= exp(ln_10*x) }
function parser_log10 ( x: real ): real;
  begin if x>0 then  parser_log10:=log10_e*ln(x) else parser_log10:=-1e35 end;
function parser_step ( x: real ): real;
  begin if x>=0 then parser_step:=1 else parser_step:=0 end;
function parser_int ( x: real ): real;
  begin parser_int:=int(x) end;
function parser_floor ( x: real ): real;
  begin if (frac(x)=0) then parser_floor:=x
   else if (x>0) then parser_floor:=trunc(x)
     else parser_floor:=-int(1-x) end;
function parser_ceil ( x: real ): real;
  begin if (frac(x)=0) then parser_ceil:=x
   else if (x>0) then parser_ceil:=trunc(1.0+x)
     else parser_ceil:=-int(-x) end;

const ParserFuncName: string[111]=
       ( 'sin'#0'cos'#0'tan'#0'log'#0'log10'#0'exp'#0'abs'#0'fabs'#0+
         'sqrt'#0'sqr'#0'asin'#0'acos'#0'atan'#0'sinh'#0'cosh'#0'tanh'#0+
         'sinc'#0'sgn'#0'step'#0'int'#0'floor'#0'ceil'#0'round'#0#0);
type rfunr = function ( x: real ):  real;
type pstring = ^string;

(* this does not work. Why??
const ParserFunc: array [0..15] of pointer =
       (sin,cos,@tan,@parser_log,@parser_log10,@exp,@fabs,@fabs,
        @sqrt,@parser_sqr,@asin,@acos,@atan,@sinh,@cosh,@tanh,
        @parser_sinc,@parser_sgn,@parser_step,@parser_int );
*)

const ParserFunc2Name: string[13] = 'pow'#0'eq'#0'mod'#0#0;
(* double (*ParserFunc2[])(double,double) =
      { @pow,@eq,@mod };
*)

function ifno0 ( a, x, y: real ): real;
  begin if a<>0 then ifno0:=x else ifno0:=y end;
function ifpos ( a, x, y: real ): real;
  begin if a>0 then ifpos:=x else ifpos:=y end;
function ifpos0 ( a, x, y: real ): real;
  begin if a>=0 then ifpos0:=x else ifpos0:=y end;
function ifneg ( a, x, y: real ): real;
  begin if a<0 then ifneg:=x else ifneg:=y end;
function ifneg0 ( a, x, y: real ): real;
  begin if a<=0 then ifneg0:=x else ifneg0:=y end;

const ParserFunc3Name: string[34] =
  'ifno0'#0'ifpos'#0'ifpos0'#0'ifneg'#0'ifneg0'#0#0;
(* double (*ParserFunc3[])(double,double,double) =
      { @ask };
*)

procedure nospaces ( var S: string {char *S} ); var I: integer;
begin
{ char *i,*j; for (i=j=S; *i; i++) if (!isspace(*i)) *j++=*i; *j=0; }
I:=Pos(' ',S);
while I<>0 do begin Delete(S,I,1); I:=Pos(' ',S) end;
end;

function strcmp ( P1: PChar; var P2: PChar ): boolean;
begin
while boolean(P1^) or boolean(P2^) do
  if P1^<>P2^ then
    begin while boolean(P2^) do inc(P2); inc(P2);
    strcmp:=false; exit end
  else begin inc(P2); inc(P1) end;
strcmp:=true;
end;

function sinh (x :real): real;
var t:real;
begin
t:=Exp(x); sinh:= 0.5*(t-1/t)
end;

function cosh (x :real): real;
var t: real;
begin t:=Exp(x); cosh:= 0.5*(t+1/t)
end;

function tanh (x :real): real;
var s,t :real;
begin s:=Exp(x); t:=1/s;
tanh:= (s-t)/(s+t); end;

function asinh (x :real): real;
var t: real;
begin
t:= x+Sqrt(Sqr(x)+1);
if t<0 then asinh:= 0.0 else asinh:= ln(t);
end;

function acosh (x :real): real;
begin if x<1.0 then acosh:=1e35 { error ! }
 else acosh:= ln(x+Sqrt(Sqr(x)-1.0));
end;

function atanh (x :real): real;
begin
if (Abs(x)>=1.0) or ((1.0-x)=0.0) then atanh:=1e35 { error ! }
 else atanh:= 0.5*Ln((1.0+x)/(1.0-x));
end;


{ remark: DoFormulaParser and ParserEvalExpr are called recursively }
function DoFormulaParser ( var Index: PChar ): real; forward;


function ParserEvalExpr ( var Index: PChar; UnaryEnabled: boolean ): real;
var Index0,Index1,Index2,TmpIndex: PChar;
    Unary,I,V,K,vari: integer;
    X,X1,X2: real; Old,Old2: char;
    label break;
begin
Unary:=1;
if (UnaryEnabled) then
  case Index^ of
    '+': inc(Index);
    '-': begin inc(Index); Unary:=-1; end
    end;
Index0:=Index;
{ read number }
if (Index^ in ['0'..'9']) or (Index^='.') then
  begin
  { X=strtod(*Index,&Index0); *Index=Index0; return Unary>0 ? X : -X; }
  V:=0; while (pchar(pointer(longint(Index)+V)))^<>#0 do inc(V);
  Old:=pchar(pointer(longint(Index)-1))^;
  pchar(pointer(longint(Index)-1))^:=char(V); { V=length(S) }
  val(pstring(pointer(longint(pointer(Index))-1))^,X,I);
  if I<>0 then
    { X=0, and length(OKstring) is known }
    begin
    V:=I-1;
    pchar(pointer(longint(Index)-1))^:=char(I-1);
    val(pstring(pointer(longint(pointer(Index))-1))^,X,I);
    end;
  pchar(pointer(longint(Index)-1))^:=Old;
  Index:=pchar(longint(Index0)+V);
  if Unary<0 then X:=-X;
  ParserEvalExpr:=X;
  exit;
  end;
{ read $n parameter }
if ParserUseDollars and (Index^='$') then begin
  inc(Index);
  if not(Index^ in ['0'..'9']) or ((pchar(longint(Index)+1))^ in ['0'..'9']) and ((pchar(longint(Index)+2))^ in ['0'..'9'])
    then begin ParserError:=peDollar; ParserEvalExpr:=0; exit end;
  i:=byte(Index^) - 48; inc(Index);
  if Index^ in ['0'..'9'] then begin i:=10*i+byte(Index^)-48; inc(Index) end;
  X:=ParserDollars^[i];
  if Unary<0 then X:=-X;
  ParserEvalExpr:=X;
  exit;
  end;
vari:=1; { nb of variables(params) in parenth. }
if (Index^<>'(') then begin
  { read name of constant or function }
  if not(Index^ in ['a'..'z','0'..'9']) then
    begin ParserError:=peIdentifier; ParserEvalExpr:=0; exit end;
  inc(Index); while Index^ in ['a'..'z','0'..'9'] do inc(Index);
  { is it x,y or z? They are equiv. to $1,$2,$3 }
  if ParserUsexyz and (longint(Index)-longint(Index0)=1) and
       (byte(Index0^)>=byte('x')) and (byte(Index0)<=byte('z') )
    then begin
      i:=byte(Index0^)-byte('w');
      X:=ParserDollars^[i];
      if Unary<0 then X:=-X;
      ParserEvalExpr:=X;
      exit;
    end;
  Old:=Index^; Index^:=#0;
  if Old<>'(' then begin { it is constant }
    i:=0; TmpIndex:=@ParserConstName[1];
    while not strcmp(Index0,TmpIndex) do
      begin inc(i);
      if not boolean(TmpIndex^) { no such constant }
        then begin ParserError:=peIdentifier; ParserEvalExpr:=0; exit end;
      end;
    X:=ParserConst[i];
    Index^:=Old;
    if Unary<0 then X:=-X;
    ParserEvalExpr:=X;
    exit;
    end;
  { it is function }
  i:=0; TmpIndex:=@ParserFuncName[1];
  while not strcmp(Index0,TmpIndex) do
   begin inc(i);
    if not boolean(TmpIndex^) { no such fun of 1 param. }
    then begin
     vari:=2; i:=0; TmpIndex:=@ParserFunc2Name[1];
     while not strcmp(Index0,TmpIndex) do
       begin inc(i);
       if not boolean(TmpIndex^) then { no such fun of 2 param. }
         begin
         vari:=3; i:=0; TmpIndex:=@ParserFunc3Name[1];
         while not strcmp(Index0,TmpIndex) do
           begin inc(i);
           if not boolean(TmpIndex^) then { no such fun of 3 param. }
             begin ParserError:=peIdentifier; ParserEvalExpr:=0; exit end;
           end;
         goto break;
         end;
       end;
     goto break;
     end;
   end;
   break:
   Index^:=Old;
   end;
{ evaluate the expression(s) in the parenthesis }
for v:=vari downto 1 do begin
Index1:=pchar(longint(Index)+1); Index2:=Index1;   { position of '('+1 }
k:=1;
while k<>0 do begin { find left parentheses or comma }
  case Index2^ of
    ')': dec(k);
    '(': inc(k);
    ',': if (k=1) and (v>1) then dec(k); { not last param }
    #0 : begin ParserError:=peParentheses; ParserEvalExpr:=0; exit end;
    end;
  inc(Index2);
  end;
dec(Index2); Old2:=Index2^; Index2^:=#0;
X:=DoFormulaParser( Index1 );
if (v=2) then X1:=X else if (v=3) then X2:=X;
Index2^:=Old2; { it was ) or , }
if (ParserError<>peOK) then begin ParserEvalExpr:=0; exit end;
Index:=Index2; inc(Index);
if (v>1) then dec(Index);
end;

if (Index0^<>'(') then { it was function and not only parenthesis }
  case (vari) of
    1: case i of { ParserFunc[i](X); break; }
       0: X:=sin(X);
       1: X:=cos(X);
       2: X:=sin(X)/cos(X);
       3: X:=parser_log(X);
       4: X:=parser_log10(X);
       5: X:=exp(X);
       6: X:=abs(X);
       7: X:=abs(X);
       8: if X<0 then X:=1e35 else X:=sqrt(X);
       9: X:=sqr(X);
       10: {asin(x)} if abs(x)>1 then X:=1e35 else if X=1 then X:=pi/2
                        else if X=-1 then X:=-pi/2 else X:=arctan (x/sqrt (1-sqr (x)));
       11: {acos(x)} if abs(X)>1 then X:=1e35 else if X=1 then X:=0
                        else if X=-1 then X:=-pi else X:=arctan (sqrt (1-sqr (x)) /x);
       12: X:=arctan(X);
       13: X:=sinh(X);
       14: X:=cosh(X);
       15: X:=tanh(X);
       16: X:=parser_sinc(abs(X));
       17: X:=parser_sgn(X);
       18: X:=parser_step(X);
       19: X:=int(X);
       20: X:=parser_floor(X); { floor }
       21: X:=parser_ceil(X);
       22: X:=round(X);
       end;
    2: case i of
       0: { pow(X1,X); X:=ParserFunc2[i](X1,X); break; }
          if X=0 then X:=1 else
            if X1<=0 then X:=0
             else X:=exp(X*ln(X1));
       1: X:=byte(X=X1);
       2: X:=X1-X*parser_floor(X1/X); { x1 mod x }
       end;
    3: case i of { X:=ParserFunc3[i](X2,X1,X); break; }
       0: X:=ifno0(X2,X1,X);
       1: X:=ifpos(X2,X1,X);
       2: X:=ifpos0(X2,X1,X);
       3: X:=ifneg(X2,X1,X);
       4: X:=ifneg0(X2,X1,X);
       end;
    end;
if Unary<0 then X:=-X;
ParserEvalExpr:=X;
end;


function DoFormulaParser ( var Index: PChar ): real;
(* routine which is called by FormulaParser or recursively via EvalExpr
   It performs  Expr1 {+= | -=} {Expr2 {{*= | /=} Expr3} } till end or error
                     ...=Oper1         ...=Oper2
*)
var Expr1, Expr2, Expr3: real;
    Oper1, Oper2: char;
    UnaryEnable: boolean;
label StopIt;
begin
Expr1:=0; Oper1:='+'; Oper2:=' '; UnaryEnable:=true;
while (Oper2<>#0) do begin
  Expr2:=ParserEvalExpr( Index, UnaryEnable );
  if (boolean(ParserError)) then begin DoFormulaParser:=0; exit end;
  UnaryEnable:=false;
  Oper2:=Index^;
  if (Oper2=#0) then goto StopIt;
  if system.Pos(Oper2,'+-*/')=0 then
    begin ParserError:=peOperator; DoFormulaParser:=Expr1; exit end;
  inc(Index);
  while Pos(Oper2,'*/')<>0 do begin
    { loop of multiplication and division }
    Expr3:=ParserEvalExpr(Index,false);
    if (boolean(ParserError)) then begin DoFormulaParser:=Expr1; exit end;
    case Oper2 of
      '*': Expr2 := Expr2*Expr3;
      '/': if (Expr3<>0) then Expr2 := Expr2/Expr3
                  else if Expr2=0 then Expr2:=1 else
                    if Expr2>0 then Expr2:=1e35 else Expr2:=1e35;
      end;
    Oper2:=Index^;
    if (Oper2=#0) then goto StopIt;
    if Pos(Oper2,'+-*/\0')=0 then
      begin ParserError:=peOperator; DoFormulaParser:=Expr1; exit end;
    inc(Index);
    end;
  { no more multiplication of division between Expr2 and Expr3,
    hence add or substract Expr1 and Expr2 }
  StopIt:
  case Oper1 of
    '+': Expr1 := Expr1+Expr2;
    '-': Expr1 := Expr1-Expr2;
    end;
  Oper1:=Oper2;
  end;
DoFormulaParser:=Expr1;
end; { DoFormulaParser }


procedure strlwr ( var S: string ); var I: integer;
begin
for I:=1 to length(S) do
  if S[I] in ['A'..'Z'] then S[I]:=char(byte(S[I])+32)
end;


procedure FormulaParserPreProcess ( var S: string );
begin
nospaces(S); strlwr(S);
end;


function FormulaParser ( var S: string ): real;
begin
ParserError:=0; ParserUsexyz:=false; ParserUseDollars:=false;
ParserIndex:=@S[1];
S[length(S)+1]:=#0;
FormulaParser:=DoFormulaParser ( ParserIndex );
end;


function FormulaParserDollars
  ( var S: string; var Dollars {: PReal} ): real;
begin
ParserError:=0; ParserUsexyz:=true; ParserUseDollars:=true;
ParserDollars:=@Dollars;
ParserIndex:=@S[1];
S[length(S)+1]:=#0;
FormulaParserDollars:=DoFormulaParser ( ParserIndex );
end;


function FormulaParserGetMaxParam ( var S: string ): integer;
var Index,Max,i: integer;
begin
Max:=0; Index:=1;
while Index<length(S) do
  begin
  if S[Index]='$' then
    begin
    inc(Index); if S[Index] in ['0'..'9'] then
      begin
      i:=byte(S[Index])-48;
      if S[Index+1] in ['0'..'9'] then
        begin inc(Index); i:=10*i+byte(S[Index])-48 end;
      end;
    if i>Max then Max:=i;
    end;
  inc(Index);
  end;
FormulaParserGetMaxParam:=Max;
end;


end. { end of PMParser.pas }
