UNIT Print;
{$F+,O+,S-,X+,D-,L-,R-}

INTERFACE

uses crt, dos, printer, graph, MujGraf;

procedure DotazNaFullHCEPSON ( Inverse: boolean; Mode: byte;
                               var TiskloSe: boolean );

procedure FullHardCopyEPSON ( Inverse: boolean; Mode: byte );
(***********************************************************************
 * Inverse: false - tisk cerne na bilem                                *
 * Mode: 1 - kvalitni tisk, pul sirky A4                               *
 *       2 - mene kvalitni na stejnem formatu                          *
 *       3 - kvalitni na ctvrtine A4                                   *
 *       4,5 - cela sirka A4, potiskne polovinu delky papiru           *
 *       6 - o neco uzsi                                               *
 *       7 - jako 4,5                                                  *
 *       atd.                                                          *
 * Papir je treba nastavit na 1.5 vroubku                              *
 ***********************************************************************)


procedure HardCopyEPSON ( OdSl, DoSl, OdRad, DoRad, LeftMarg10 :integer );
  (**********************************************************************+
  |  Provadi HardCopy pro tiskarnu EPSON v  graphics print mode I        |
  |  bez pouziti unit Printer primo pres sluzbu DOSu                     |
  |  Verse 1.0  / 12-04-89                                               |
  |  HardCopy na tiskarne EPSON FX-800.                                  |
  |  HardCopy OdSl(oupce)  DoSl(oupce) a OdRad(ku) DoRad(ku)             |
  |   - udaje v jednotkach CELE obrazovky !                              |
  |  LeftMarg10 = odsazeni od leveho okraje po deseti sloupcich,         |
  | ---------------------------------------------------------------------|
  |  Pro OdSl  = DoSl   se dosadi OdSl := 0; DoSl := GetMaxX             |
  |      OdRad = DoRad    - " -   OdRad:= 0; DoRad:= GetMaxY             |
  +**********************************************************************)

{ VYBLOKOVANA
procedure HardCopyD100 ( OdSl, DoSl, OdRad, DoRad, LeftMarg10: integer;
                         CondPrint, DoubleDens: boolean );
  (**********************************************************************+
  |  HardCopy na tiskarne D100.                                          |
  |  HardCopy OdSl(oupce)  DoSl(oupce) a OdRad(ku) DoRad(ku)             |
  |   - udaje v jednotkach CELE obrazovky !                              |
  |  LeftMarg10 = odsazeni od leveho okraje po deseti sloupcich,         |
  |  CondPrint  = True/False (zhusteny tisk sloupcu ano/ne)              |
  |  DoubleDens = True/False (dvojity tisk ano/ne)                       |
  | ---------------------------------------------------------------------|
  |  Pro OdSl  = DoSl   se dosadi OdSl := 0; DoSl := GetMaxX             |
  |      OdRad = DoRad    - " -   OdRad:= 0; DoRad:= GetMaxY             |
  +**********************************************************************)
}

var VP_col: integer;


IMPLEMENTATION

procedure FullHardCopyEPSON(Inverse:boolean;Mode:byte); { EPSON }
  var I,J,top:integer;
      XScreenMaxGlb,YMaxGlb: integer;
      BkColor,PrintByte:byte;

  procedure DoLine(top:integer);
    var J:integer;
    function ConstructByte(J,I:integer):byte;
      const Bits:array [0..7] of byte=(128,64,32,16,8,4,2,1);
      var CByte,K:byte;
      begin
        I:=I shl 3;
        CByte:=0;
        for K:=0 to top do
          if GetPixel(J,I+K)<>BkColor then CByte:=CByte or Bits[K];
        ConstructByte:=CByte;
      end {ConstructByte};
    begin
      if keypressed and (ReadKey=#$1B) then exit;
      if Mode=1 then write(lst,^['L')
      else write(lst,^['*',chr(Mode));
      write(lst,chr(lo(XScreenMaxGlb+1)),chr(Hi(XScreenMaxGlb+1)));
      for J:=0 to XScreenMaxGlb do
       begin
        PrintByte:=ConstructByte(J,I);
        if Inverse then PrintByte:=not PrintByte;
        write(lst,chr(PrintByte));
       end;
      if Mode<>4 then writeln(lst);
    end {DoLine};

  begin
   XScreenMaxGlb:= GetMaxX;
   YMaxGlb:= GetMaxY;
   BkColor:=GetBkColor;
   top:=7;
   Mode:=Mode and 7;
   if (Mode=5) or (Mode=0) then Mode:=4;
   write(lst,^['3'#24);
   for I:=0 to ((YMaxGlb+1) shr 3)-1 do
     begin if keypressed and (ReadKey=#$1B) then exit;
     DoLine(7) end;
   I:=((YMaxGlb+1) shr 3);
   if (YMaxGlb+1) and 7<>0 then
      DoLine((YMaxGlb+1) and 7);
   writeln(lst,^['2');
  end {FullHardCopy};


procedure HardCopyEPSON (OdSl,DoSl,OdRad,DoRad,LeftMarg10 :integer);
 const
  ESC:byte = 27;
  tri:byte = 51;  (* '3' *)
  male_l:byte = 108;
  velke_L:byte = 76;
  Mask :array [0..7] of byte = (1,2,4,8,16,32,64,128);
  radkovani:byte = 24;
 var
  I,J,K,kBK,radek,bit,DoRad1,DoRad8 :integer;
  osm,PocetSl1,PocetSl2,N :byte;
  PrtRad :array [0..640] of byte;
  ViewPort :ViewPortType;

procedure SendByte(B : byte);
{ Send one byte to the printer }
const
  LPTPortNum = 1; { Defaults to LPT1. 2 = LPT2 }
var
  qq:byte;
  Regs : Registers;
begin
  Regs.AH := 0;
  Regs.Al := B;
  Regs.DX := Pred(LPTPortNum);
  Intr($17, Regs);
  qq:=(Regs.AH and 9);
  if (Regs.AH and 9)=9 then
    begin
      sound(220);delay(250);nosound;
      delay(5000);
      sound(440);delay(250);nosound;
      Regs.AH := 0;
      Regs.Al := B;
      Regs.DX := Pred(LPTPortNum);
      Intr($17, Regs);
      if (Regs.AH and 9)=9 then
        begin
          sound(880);delay(250);nosound;
          RunError(160);
        end;
    end;
end; { SendByte }

begin   (* HardCopyEPSON *)
 GetViewSettings(ViewPort);
 SetViewPort(0,0,GetMaxX,GetMaxY,ClipOff);
 if OdSl=DoSl then begin OdSl:=0; DoSl:=GetMaxX end;
 if OdRad=DoRad then begin OdRad:=0; DoRad:=GetMaxY end;
 kBK:=GetBkColor;
 PocetSl1:=(DoSl-OdSl+1) mod 256;
 PocetSl2:=trunc((DoSl-OdSl+1)/256);
 SendByte (ESC);SendByte (tri);SendByte (radkovani);
                                         (* Radkovani 24/216 inch *)
 radek:=OdRad; DoRad1:=DoRad+1; DoRad8:=DoRad+8;
 repeat
 if keypressed and (ReadKey=#$1B) then exit;
  for I:=OdSl to DoSl do
   begin
   osm:=$FF; bit:=7; J:=radek;
   repeat
    K:=GetPixel(I,J);
    if K<>kBK then
       osm:=osm and (not Mask[bit]);
    dec(bit); inc(J); PrtRad[I]:=osm;
   until (bit=-1) or (J=DoRad1);
  end  (* for i *);
  N:=LeftMarg10;
  if LeftMarg10>0 then
   begin
    SendByte (ESC);SendByte(male_l);SendByte(N);
   end;
  for I:=OdSl to DoSl do PrtRad[I]:=255-PrtRad[I];
  SendByte (ESC);SendByte (42); SendByte(4);  (* Graficky mod I *)
  SendByte (PocetSl1);
  SendByte (PocetSl2);
  for I:=OdSl to DoSl do
   SendByte (PrtRad[I]);
  SendByte (13);SendByte (10);  (* CR LF...novy radek *)
  inc(radek,8);
 until (radek>DoRad8);
   SendByte (ESC);SendByte (64);       (* Inicializace tiskarny *)
   with ViewPort do SetViewPort(X1,Y1,X2,Y2,Clip);
end  (* HardCopyEPSON *);


{
procedure HardCopyD100 (OdSl,DoSl,OdRad,DoRad,LeftMarg10 :integer;
                        CondPrint,DoubleDens :boolean);
 const
  ESC = #27;
  Mask :array [0..7] of byte = (1,2,4,8,16,32,64,128);
 var
  I,J,K,kBK,radek,bit,DoRad1,DoRad7 :integer;
  sedm :byte;
  PrtRad :array [0..640] of byte;
  ViewPort :ViewPortType;

begin   (* HardCopy *)
 GetViewSettings(ViewPort);
 SetViewPort(0,0,GetMaxX,GetMaxY,ClipOff);
 if OdSl=DoSl then begin OdSl:=0; DoSl:=GetMaxX end;
 if OdRad=DoRad then begin OdRad:=0; DoRad:=GetMaxY end;
 kBK:=GetBkColor;
 writeln(lst,ESC,'0',ESC,'1',ESC,'7');
                                 (*Radkovani 10,graf.rezim,smer ->*)
 if CondPrint then writeln(lst,$F);              (* Zhusteny tisk *)
 if DoubleDens then writeln(lst,ESC,'E');     (* Dvojnasobny tisk *)
 radek:=OdRad; DoRad1:=DoRad+1; DoRad7:=DoRad+7;
 repeat
  for I:=OdSl to DoSl do
   begin
   sedm:=$FF; bit:=6; J:=radek;
   repeat
    K:=GetPixel(I,J);
    if (K<>VP_col) and (K<>kBK) then
       sedm:=sedm and (not Mask[bit]);
    dec(bit); inc(J); PrtRad[I]:=sedm;
   until (bit=-1) or (J=DoRad1);
   end  (* for i *);
  if LeftMarg10>0 then write(lst,chr(9),chr(LeftMarg10));
  for I:=OdSl to DoSl do write(lst,chr(PrtRad[I])); writeln(lst);
  inc(radek,7);
 until (radek>DoRad7);
 writeln(lst,ESC,'5',ESC,'3',ESC,'2');
                       (* Zrus graf.rezim a smer->,radkovani 6 *)
 if CondPrint then writeln(lst,$12);      (* Rusi zhusteny tisk *)
 if DoubleDens then writeln(lst,ESC,'F'); (* Rusi dvojity tisk *)
 with ViewPort do SetViewPort(X1,Y1,X2,Y2,Clip);
end  (* HardCopyD100 *);
}

procedure DotazNaFullHCEPSON ( Inverse: boolean; Mode: byte; var TiskloSe: boolean );
var C: char; const S='Vytisknout obrazek na tiskarne [A=ano]?';
                   M='                                       ';
begin
iplttext(S,(GetMaxX-TextWidth(S)) div 2, trunc(GetMaxY*0.92),1,3,red,black);
repeat until KeyPressed; TiskloSe:=UpCase(ReadKey)='A';
iplttext(M,(GetMaxX-TextWidth(S)) div 2,trunc(GetMaxY*0.92),1,3,red,black);
if TiskloSe then FullHardCopyEPSON(Inverse,Mode)
end;


end.