UNIT Vstupy;
{ Copyright (c) 1990 by Borland International }
{               1991 Petr Mikulk             }


{$F+,O+,S-,X+,D-,L-,R-}

INTERFACE

uses Objects, Drivers, Dialogs;

type

  { Same as TInputLine, except invalid if empty }
  PKeyInputLine = ^TKeyInputLine;
  TKeyInputLine = object(TInputLine)
    function Valid(Command: Word): boolean; virtual;
  end;

  { Accepts only valid numeric input between Min and Max }
  PNumInputLine = ^TNumInputLine;
  TNumInputLine = object(TInputLine)
    Min: longint;
    Max: longint;
    constructor Init(var Bounds: TRect; AMaxLen: integer;
      AMin, AMax: longint);
    constructor Load(var S: TStream);
    function DataSize: Word; virtual;
    procedure GetData(var Rec); virtual;
    procedure SetData(var Rec); virtual;
    procedure Store(var S: TStream);
    function Valid(Command: Word): boolean; virtual;
  end;

  { Accepts only valid numeric input between Min and Max, which }
  { are Real numbers }
  PRealInputLine = ^TRealInputLine;
  TRealInputLine = object(TInputLine)
    Min,Max: real;
    Mist,Des: integer; { pro vstup z SetData }
    constructor Init(var Bounds: TRect; AMaxLen: integer;
      AMin, AMax: real; AMist, ADes: integer );
    constructor Load(var S: TStream);
    function DataSize: Word; virtual;
    procedure GetData(var Rec); virtual;
    procedure SetData(var Rec); virtual;
    procedure Store(var S: TStream);
    function Valid(Command: Word): boolean; virtual;
  end;

procedure RegisterFields;

const
  RKeyInputLine: TStreamRec = (
     ObjType: 10060;
     VmtLink: Ofs(TypeOf(TKeyInputLine)^);
     Load:    @TKeyInputLine.Load;
     Store:   @TKeyInputLine.Store
  );
  RNumInputLine: TStreamRec = (
     ObjType: 10061;
     VmtLink: Ofs(TypeOf(TNumInputLine)^);
     Load:    @TNumInputLine.Load;
     Store:   @TNumInputLine.Store
  );

IMPLEMENTATION

uses Views, ZprBox;

procedure RegisterFields;
begin
  RegisterType(RKeyInputLine);
  RegisterType(RNumInputLine);
end;

{ TKeyInputLine }

function TKeyInputLine.Valid(Command: Word): boolean;
var
  Ok: boolean;
begin
  Ok := true;
  if (Command <> cmCancel) and (Command <> cmValid) then
  begin
    if Data^ = '' then
    begin
      Select;
      ZpravaBox('','This field cannot be empty.', nil, mfError + mfOkButton);
      Ok := false;
    end;
  end;
  if Ok then Valid := TInputLine.Valid(Command)
  else Valid := false;
end;

{ TNumInputLine }
constructor TNumInputLine.Init(var Bounds: TRect; AMaxLen: integer;
  AMin, AMax: longint);
begin
  TInputLine.Init(Bounds, AMaxLen);
  Min := AMin;
  Max := AMax;
end;

constructor TNumInputLine.Load(var S: TStream);
begin
  TInputLine.Load(S);
  S.read(Min, SizeOf(longint) * 2);
end;

function TNumInputLine.DataSize: Word;
begin
  DataSize := SizeOf(longint);
end;

procedure TNumInputLine.GetData(var Rec);
var
  Code: integer;
begin
  val(Data^, longint(Rec), Code);
end;

procedure TNumInputLine.Store(var S: TStream);
begin
  TInputLine.Store(S);
  S.write(Min, SizeOf(longint) * 2);
end;

procedure TNumInputLine.SetData(var Rec);
var
  S: string;
begin
  str(longint(Rec), S );
  if length(S) <= MaxLen then Data^:=S
    else Data^:=Copy(S,1,MaxLen-1)+'!';
  SelectAll(true);
end;

function TNumInputLine.Valid(Command: Word): boolean;
var
  Code: integer;
  Value: longint;
  Params: array[0..1] of longint;
  Ok: boolean;
begin
  Ok := true;
  if (Command <> cmCancel) and (Command <> cmValid) then
  begin
    if Data^ = '' then Data^ := '0';
    val(Data^, Value, Code);
    if (Code <> 0) or (Value < Min) or (Value > Max) then
    begin
      Select;
      Params[0] := Min;
      Params[1] := Max;
      ZpravaBox('','Number must be from '+'%D to %D', @Params, mfError + mfOkButton);
      SelectAll(true);
      Ok := false;
    end;
  end;
  if Ok then Valid := TInputLine.Valid(Command)
  else Valid := false;
end;

{ TRealInputLine }
constructor TRealInputLine.Init(var Bounds: TRect; AMaxLen: integer;
  AMin, AMax: real; AMist, ADes: integer );
begin
  TInputLine.Init(Bounds, AMaxLen);
  Min := AMin; Max := AMax; Mist:=AMist; Des:=ADes;
end;

constructor TRealInputLine.Load(var S: TStream);
begin
  TInputLine.Load(S);
  S.read(Min, SizeOf(real) * 2 + SizeOf(integer)*2 );
end;

function TRealInputLine.DataSize: Word;
begin
  DataSize := SizeOf(real);
end;

procedure TRealInputLine.GetData(var Rec);
var Code: integer;
begin
  val(Data^, real(Rec), Code);
end;

procedure TRealInputLine.Store(var S: TStream);
begin
  TInputLine.Store(S);
  S.write(Min, SizeOf(real) * 2 + SizeOf(integer)*2 );
end;

procedure TRealInputLine.SetData(var Rec);
var S: string; Pom: real; Code: integer;
begin
str(real(Rec):Mist:Des, S);
  val(S, Pom, Code);
  if Pom<>real(Rec) then
    begin str(real(Rec):Mist,S); val(S, Pom, Code);
      if Pom<>real(Rec) then str(real(Rec),S);
    end;
if Length(S) <= MaxLen
  then Data^:=S
  else
    begin str(real(Rec):MaxLen,S);
    if Length(S) <= MaxLen then Data^:=S
      else Data^:='!' end;
SelectAll(true)
end;

function TRealInputLine.Valid(Command: Word): boolean;
var
  Code: integer;
  Value: real;
  S1,S2: String[25];
  Ok: boolean;
begin
  Ok := true;
  if (Command <> cmCancel) and (Command <> cmValid) then
  begin
    if Data^ = '' then Data^ := '0';
    val(Data^, Value, Code);
    if (Code <> 0) or (Value < Min) or (Value > Max) then
    begin
      Select;
      Str(Min,S1); Str(Max,S2);
      ZpravaBox('','Number must be'+#$D'from'+S1+#$D'to   '+S2, nil, mfError + mfOkButton);
      SelectAll(true);
      Ok := false;
    end;
  end;
  if Ok then Valid := TInputLine.Valid(Command)
  else Valid := false;
end;

end.
