{*******************************************************}
{                                                       }
{       Turbo Pascal Version 6.0                        }
{       Turbo Vision Unit                               }
{                                                       }
{       Copyright (c) 1990 Borland International        }
{                                                       }
{    Upraveno do ceske verze - PM                       }
{*******************************************************}

UNIT ZprBox;

{$F+,O+,S-,X+,D-,L-,R-}

INTERFACE

uses Objects, App1;

const

{ Message box classes }

  mfError        = $0001;       { Display a Error box }
  mfInformation  = $0002;       { Display an Information Box }
  mfConfirmation = $0003;       { Display a Confirmation Box }
  mfWarning      = $0004;       { Display a Warning box }

{ Message box button flags }

  mfYesButton    = $0100;       { Put a Yes button into the dialog }
  mfNoButton     = $0200;       { Put a No button into the dialog }
  mfOkButton     = $0400;       { Put an OK button into the dialog }
  mfCancelButton = $0800;       { Put a Cancel button into the dialog }

  mfYesNoCancel  = mfYesButton + mfNoButton + mfCancelButton;
                                { Standard Yes, No, Cancel dialog }
  mfOKCancel     = mfOkButton + mfCancelButton;
                                { Standard OK, Cancel dialog }
{ !!! PRIDAL JSEM: }
  mfNoDefault    = $1000;  { implicitni se bere posledni polozka }
  mfYesNo        = mfYesButton + mfNoButton;
{ !!! }
{ ZpravaBox displays the given string in a standard sized      }
{ dialog box. Before the dialog is displayed the Msg and Params }
{ are passed to FormatStr.  The resulting string is displayed   }
{ as a TStaticText view in the dialog.                          }

function ZpravaBox(Title,Msg: string; Params: Pointer; AOptions: Word): Word;

{ ZpravaBoxRec allows the specification of a TRect for the     }
{ message box to occupy.                                        }

function ZpravaBoxRect(var R: TRect; Title,Msg: string; Params: Pointer;
  AOptions: Word): Word;

{ InputBox displays a simple dialog that allows the user to     }
{ type in a string.                                             }

function InputBox(Title: string; ALabel: string; var S: string;
  Limit: byte): Word;

{ InputBoxRect is like InputBox but allows the specification of }
{ a rectangle.                                                  }

function InputBoxRect(var Bounds: TRect; Title: string; ALabel: string;
  var S: string;  Limit: byte): Word;

IMPLEMENTATION

uses Drivers, Views, Dialogs;

function ZpravaBox(Title,Msg: string; Params: Pointer; AOptions: Word): Word;
var
  R: TRect;
begin
  R.assign(0, 0, 40, 9);
  R.Move((DeskTop^.Size.X - R.B.X) div 2, (DeskTop^.Size.Y - R.B.Y) div 2);
  ZpravaBox := ZpravaBoxRect(R, Title, Msg, Params, AOptions);
end;

function ZpravaBoxRect(var R: TRect; Title,Msg: string; Params: Pointer;
  AOptions: Word): Word;
const
  ButtonName: array[0..3] of string[6] =
    ('~Y~es', '~N~o', 'O~K~', 'Cancel');
  Commands: array[0..3] of Word =
    (cmYes, cmNo, cmOK, cmCancel);
  Titles: array[1..4] of string[11] =
    ('Error','Information','Confirmation','Warning');
var
  I, X, ButtonCount: integer;
  Dialog: PDialog;
  Control: PView;
  T: TRect;
  ButtonList: array[0..4] of PView;
  S: string;
begin
  if (AOptions and $00FF) in [1..4] then Title:=Titles[AOptions and $00FF]+' '+Title;
  Dialog := new(PDialog,Init(R,Title));
  with Dialog^ do
  begin
    R.assign(3, 2, Size.X - 2, Size.Y - 3);
    FormatStr(S, Msg, Params^);
    Control := new(PStaticText, Init(R, S));
    Insert(Control);
    X := -2;
    ButtonCount := 0;
    for I := 0 to 3 do
      if AOptions and ($0100 shl I) <> 0 then
      begin
        R.assign(0, 0, 10, 2);
        Control := new(PButton, Init(R, ButtonName[I], Commands[I],
          bfNormal));
        inc(X, Control^.Size.X + 2);
        ButtonList[ButtonCount] := Control;
        inc(ButtonCount);
      end;
    X := (Size.X - X) shr 1;
    for I := 0 to ButtonCount - 1 do
    begin
      Control := ButtonList[I];
      Insert(Control);
      Control^.MoveTo(X, Size.Y - 3);
      inc(X, Control^.Size.X + 2);
    end;
{ !!! PRIDAL JSEM : }
   IF AOptions and mfNoDefault = 0 then
{ !!! }
    SelectNext(false);
  end;
  ZpravaBoxRect := DeskTop^.ExecView(Dialog);
  dispose(Dialog, Done);
end;

function InputBox(Title: string; ALabel: string; var S: string;
  Limit: byte): Word;
var
  R: TRect;
begin
  R.assign(0, 0, 60, 8);
  R.Move((DeskTop^.Size.X - R.B.X) div 2, (DeskTop^.Size.Y - R.B.Y) div 2);
  InputBox := InputBoxRect(R, Title, ALabel, S, Limit);
end;

function InputBoxRect(var Bounds: TRect; Title: string; ALabel: string;
  var S: string;  Limit: byte): Word;
var
  Dialog: PDialog;
  Control: PView;
  R: TRect;
  C: Word;
begin
  Dialog := new(PDialog, Init(Bounds, Title));
  with Dialog^ do
  begin
    R.assign(4 + CStrLen(ALabel), 2, Size.X - 3, 3);
    Control := new(PInputLine, Init(R, Limit));
    Insert(Control);
    R.assign(2, 2, 3 + CStrLen(ALabel), 3);
    Insert(new(PLabel, Init(R, ALabel, Control)));
    R.assign(Size.X - 24, Size.Y - 4, Size.X - 14, Size.Y - 2);
    Insert(new(PButton, Init(R, 'O~K~', cmOK, bfDefault)));
    inc(R.A.X, 12); inc(R.B.X, 12);
    Insert(new(PButton, Init(R, 'Zruit', cmCancel, bfNormal)));
    inc(R.A.X, 12); inc(R.B.X, 12);
    SelectNext(false);
  end;
  Dialog^.SetData(S);
  C := DeskTop^.ExecView(Dialog);
  if C <> cmCancel then Dialog^.GetData(S);
  dispose(Dialog, Done);
  InputBoxRect := C;
end;

end.
